@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
 #include <string.h>
@@ -8,17 +8,18 @@
 #include "cap-list.h"
 #include "extract-word.h"
 #include "macro.h"
-#include "missing.h"
 #include "parse-util.h"
+#include "stdio-util.h"
 #include "util.h"
 
+#if 0 /// UNNEEDED by elogind
 static const struct capability_name* lookup_capability(register const char *str, register GPERF_LEN_TYPE len);
 
 #include "cap-from-name.h"
+#endif // 0
 #include "cap-to-name.h"
 
 const char *capability_to_name(int id) {
-
         if (id < 0)
                 return NULL;
 
@@ -28,6 +29,7 @@ const char *capability_to_name(int id) {
         return capability_names[id];
 }
 
+#if 0 /// UNNEEDED by elogind
 int capability_from_name(const char *name) {
         const struct capability_name *sc;
         int r, i;
@@ -37,7 +39,7 @@ int capability_from_name(const char *name) {
         /* Try to parse numeric capability */
         r = safe_atoi(name, &i);
         if (r >= 0) {
-                if (i >= 0 && (size_t) i < ELEMENTSOF(capability_names))
+                if (i >= 0 && i < 64)
                         return i;
                 else
                         return -EINVAL;
@@ -51,36 +53,42 @@ int capability_from_name(const char *name) {
         return sc->id;
 }
 
+/* This is the number of capability names we are *compiled* with.
+ * For the max capability number of the currently-running kernel,
+ * use cap_last_cap(). */
 int capability_list_length(void) {
         return (int) ELEMENTSOF(capability_names);
 }
+#endif // 0
 
 int capability_set_to_string_alloc(uint64_t set, char **s) {
         _cleanup_free_ char *str = NULL;
-        unsigned long i;
-        size_t allocated = 0, n = 0;
+        size_t n = 0;
 
         assert(s);
 
-        for (i = 0; i <= cap_last_cap(); i++)
+        for (unsigned i = 0; i <= cap_last_cap(); i++)
                 if (set & (UINT64_C(1) << i)) {
                         const char *p;
+                        char buf[2 + 16 + 1];
                         size_t add;
 
                         p = capability_to_name(i);
-                        if (!p)
-                                return -EINVAL;
+                        if (!p) {
+                                xsprintf(buf, "0x%x", i);
+                                p = buf;
+                        }
 
                         add = strlen(p);
 
-                        if (!GREEDY_REALLOC(str, allocated, n + add + 2))
+                        if (!GREEDY_REALLOC(str, n + add + 2))
                                 return -ENOMEM;
 
                         strcpy(mempcpy(str + n, p, add), " ");
                         n += add + 1;
                 }
 
-        if (!GREEDY_REALLOC(str, allocated, n + 1))
+        if (!GREEDY_REALLOC(str, n + 1))
                 return -ENOMEM;
 
         str[n > 0 ? n - 1 : 0] = '\0'; /* truncate the last space, if it's there */
@@ -90,13 +98,13 @@ int capability_set_to_string_alloc(uint64_t set, char **s) {
         return 0;
 }
 
+#if 0 /// UNNEEDED by elogind
 int capability_set_from_string(const char *s, uint64_t *set) {
         uint64_t val = 0;
-        const char *p;
 
         assert(set);
 
-        for (p = s;;) {
+        for (const char *p = s;;) {
                 _cleanup_free_ char *word = NULL;
                 int r;
 
@@ -117,3 +125,4 @@ int capability_set_from_string(const char *s, uint64_t *set) {
 
         return 0;
 }
+#endif // 0
