@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
 #include <alloca.h>
@@ -42,35 +42,53 @@
 #  define DEFAULT_USER_PATH DEFAULT_PATH
 #endif
 
-bool is_path(const char *p) _pure_;
+#if 0 /// UNNEEDED by elogind
+static inline bool is_path(const char *p) {
+        if (!p) /* A NULL pointer is definitely not a path */
+                return false;
+
+        return strchr(p, '/');
+}
+#endif // 0
+
+static inline bool path_is_absolute(const char *p) {
+        if (!p) /* A NULL pointer is definitely not an absolute path */
+                return false;
+
+        return p[0] == '/';
+}
+
 #if 0 /// UNNEEDED by elogind
 int path_split_and_make_absolute(const char *p, char ***ret);
-#endif // 0
-bool path_is_absolute(const char *p) _pure_;
-#if 0 /// UNNEEDED by elogind
 char* path_make_absolute(const char *p, const char *prefix);
 #endif // 0
 int safe_getcwd(char **ret);
 int path_make_absolute_cwd(const char *p, char **ret);
 #if 0 /// UNNEEDED by elogind
-int path_make_relative(const char *from_dir, const char *to_path, char **_r);
+int path_make_relative(const char *from, const char *to, char **ret);
+int path_make_relative_parent(const char *from_child, const char *to, char **ret);
 #endif // 0
-char* path_startswith(const char *path, const char *prefix) _pure_;
+char *path_startswith_full(const char *path, const char *prefix, bool accept_dot_dot) _pure_;
+static inline char* path_startswith(const char *path, const char *prefix) {
+        return path_startswith_full(path, prefix, true);
+}
 int path_compare(const char *a, const char *b) _pure_;
-bool path_equal(const char *a, const char *b) _pure_;
+
+static inline bool path_equal(const char *a, const char *b) {
+        return path_compare(a, b) == 0;
+}
+
 bool path_equal_or_files_same(const char *a, const char *b, int flags);
-char* path_join_internal(const char *first, ...);
-#define path_join(x, ...) path_join_internal(x, __VA_ARGS__, (const char*) -1)
+#if 0 /// UNNEEDED by elogind
+/* Compares only the last portion of the input paths, ie: the filenames */
+bool path_equal_filename(const char *a, const char *b);
+#endif // 0
 
-char* path_simplify(char *path, bool kill_dots);
+char* path_extend_internal(char **x, ...);
+#define path_extend(x, ...) path_extend_internal(x, __VA_ARGS__, POINTER_MAX)
+#define path_join(...) path_extend_internal(NULL, __VA_ARGS__, POINTER_MAX)
 
-enum {
-        PATH_CHECK_FATAL    = 1 << 0,  /* If not set, then error message is appended with 'ignoring'. */
-        PATH_CHECK_ABSOLUTE = 1 << 1,
-        PATH_CHECK_RELATIVE = 1 << 2,
-};
-
-int path_simplify_and_warn(char *path, unsigned flag, const char *unit, const char *filename, unsigned line, const char *lvalue);
+char* path_simplify(char *path);
 
 static inline bool path_equal_ptr(const char *a, const char *b) {
         return !!a == !!b && (!a || path_equal(a, b));
@@ -88,13 +106,16 @@ int path_strv_make_absolute_cwd(char **l);
 char** path_strv_resolve(char **l, const char *root);
 char** path_strv_resolve_uniq(char **l, const char *root);
 
-int find_binary(const char *name, char **filename);
-
 #if 0 /// UNNEEDED by elogind
+int find_executable_full(const char *name, const char *root, char **exec_search_path, bool use_path_envvar, char **ret_filename, int *ret_fd);
+static inline int find_executable(const char *name, char **ret_filename) {
+        return find_executable_full(name, /* root= */ NULL, NULL, true, ret_filename, NULL);
+}
+
 bool paths_check_timestamp(const char* const* paths, usec_t *paths_ts_usec, bool update);
 
-int fsck_exists(const char *fstype);
-int mkfs_exists(const char *fstype);
+int fsck_exists(void);
+int fsck_exists_for_fstype(const char *fstype);
 #endif // 0
 
 /* Iterates through the path prefixes of the specified path, going up
@@ -102,7 +123,7 @@ int mkfs_exists(const char *fstype);
  * directory. Excludes the specified directory itself */
 #define PATH_FOREACH_PREFIX(prefix, path)                               \
         for (char *_slash = ({                                          \
-                                path_simplify(strcpy(prefix, path), false); \
+                                path_simplify(strcpy(prefix, path));    \
                                 streq(prefix, "/") ? NULL : strrchr(prefix, '/'); \
                         });                                             \
              _slash && ((*_slash = 0), true);                           \
@@ -111,7 +132,7 @@ int mkfs_exists(const char *fstype);
 /* Same as PATH_FOREACH_PREFIX but also includes the specified path itself */
 #define PATH_FOREACH_PREFIX_MORE(prefix, path)                          \
         for (char *_slash = ({                                          \
-                                path_simplify(strcpy(prefix, path), false); \
+                                path_simplify(strcpy(prefix, path));    \
                                 if (streq(prefix, "/"))                 \
                                         prefix[0] = 0;                  \
                                 strrchr(prefix, 0);                     \
@@ -144,19 +165,25 @@ int mkfs_exists(const char *fstype);
                 _ret;                                                   \
         })
 
-#if 0 /// UNNEEDED by elogind
-int parse_path_argument_and_warn(const char *path, bool suppress_root, char **arg);
-#endif // 0
-
-char* dirname_malloc(const char *path);
+int path_find_first_component(const char **p, bool accept_dot_dot, const char **ret);
+int path_find_last_component(const char *path, bool accept_dot_dot, const char **next, const char **ret);
 const char *last_path_component(const char *path);
-int path_extract_filename(const char *p, char **ret);
+int path_extract_filename(const char *path, char **ret);
+int path_extract_directory(const char *path, char **ret);
 
 bool filename_is_valid(const char *p) _pure_;
-bool path_is_valid(const char *p) _pure_;
+bool path_is_valid_full(const char *p, bool accept_dot_dot) _pure_;
+static inline bool path_is_valid(const char *p) {
+        return path_is_valid_full(p, /* accept_dot_dot= */ true);
+}
+static inline bool path_is_safe(const char *p) {
+        return path_is_valid_full(p, /* accept_dot_dot= */ false);
+}
 bool path_is_normalized(const char *p) _pure_;
 
+#if 0 /// UNNEEDED by elogind
 char *file_in_same_dir(const char *path, const char *filename);
+#endif // 0
 
 bool hidden_or_backup_file(const char *filename) _pure_;
 
@@ -165,8 +192,6 @@ bool is_device_path(const char *path);
 
 bool valid_device_node_path(const char *path);
 bool valid_device_allow_pattern(const char *path);
-
-int systemd_installation_has_version(const char *root, unsigned minimal_version);
 #endif // 0
 
 bool dot_or_dot_dot(const char *path);
@@ -181,7 +206,7 @@ static inline const char *skip_dev_prefix(const char *p) {
         return e ?: p;
 }
 
-bool empty_or_root(const char *root);
+bool empty_or_root(const char *path);
 static inline const char *empty_to_root(const char *path) {
         return isempty(path) ? "/" : path;
 }
@@ -189,4 +214,6 @@ static inline const char *empty_to_root(const char *path) {
 bool path_strv_contains(char **l, const char *path);
 #if 0 /// UNNEEDED by elogind
 bool prefixed_path_strv_contains(char **l, const char *path);
+
+int path_glob_can_match(const char *pattern, const char *prefix, char **ret);
 #endif // 0
