@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <stdbool.h>
 #include <stddef.h>
@@ -11,9 +11,9 @@
 #include "parse-util.h"
 #include "proc-cmdline.h"
 #include "process-util.h"
-#include "special.h"
+//#include "special.h"
 #include "string-util.h"
-#include "util.h"
+//#include "util.h"
 #include "virt.h"
 
 int proc_cmdline(char **ret) {
@@ -47,7 +47,7 @@ static int proc_cmdline_extract_first(const char **p, char **ret_word, ProcCmdli
                 _cleanup_free_ char *word = NULL;
                 const char *c;
 
-                r = extract_first_word(&q, &word, NULL, EXTRACT_UNQUOTE|EXTRACT_RELAX);
+                r = extract_first_word(&q, &word, NULL, EXTRACT_UNQUOTE|EXTRACT_RELAX|EXTRACT_RETAIN_ESCAPE);
                 if (r < 0)
                         return r;
                 if (r == 0)
@@ -124,9 +124,7 @@ int proc_cmdline_parse(proc_cmdline_parse_t parse_item, void *data, ProcCmdlineF
 
         /* We parse the EFI variable first, because later settings have higher priority. */
 
-        r = elogind_efi_options_variable(&line);
         if (!FLAGS_SET(flags, PROC_CMDLINE_IGNORE_EFI_OPTIONS)) {
-                r = elogind_efi_options_variable(&line);
                 r = elogind_efi_options_variable(&line);
                 if (r < 0) {
                         if (r != -ENODATA)
@@ -272,7 +270,6 @@ int proc_cmdline_get_key(const char *key, ProcCmdlineFlags flags, char **ret_val
 
         line = mfree(line);
         r = elogind_efi_options_variable(&line);
-        r = elogind_efi_options_variable(&line);
         if (r == -ENODATA) {
                 if (ret_value)
                         *ret_value = NULL;
