@@ -1,15 +1,17 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
 #include <stdbool.h>
 
+#include "errno-list.h"
+#include "glyph-util.h"
 #include "macro.h"
 
 /* The enum order is used to order unit jobs in the job queue
  * when other criteria (cpu weight, nice level) are identical.
  * In this case service units have the highest priority. */
 typedef enum UnitType {
-        UNIT_SERVICE = 0,
+        UNIT_SERVICE,
         UNIT_MOUNT,
         UNIT_SWAP,
         UNIT_SOCKET,
@@ -21,11 +23,12 @@ typedef enum UnitType {
         UNIT_SLICE,
         UNIT_SCOPE,
         _UNIT_TYPE_MAX,
-        _UNIT_TYPE_INVALID = -1
+        _UNIT_TYPE_INVALID = -EINVAL,
+        _UNIT_TYPE_ERRNO_MAX = -ERRNO_MAX, /* Ensure the whole errno range fits into this enum */
 } UnitType;
 
 typedef enum UnitLoadState {
-        UNIT_STUB = 0,
+        UNIT_STUB,
         UNIT_LOADED,
         UNIT_NOT_FOUND,    /* error condition #1: unit file not found */
         UNIT_BAD_SETTING,  /* error condition #2: we couldn't parse some essential unit file setting */
@@ -33,7 +36,7 @@ typedef enum UnitLoadState {
         UNIT_MERGED,
         UNIT_MASKED,
         _UNIT_LOAD_STATE_MAX,
-        _UNIT_LOAD_STATE_INVALID = -1
+        _UNIT_LOAD_STATE_INVALID = -EINVAL,
 } UnitLoadState;
 
 typedef enum UnitActiveState {
@@ -45,7 +48,7 @@ typedef enum UnitActiveState {
         UNIT_DEACTIVATING,
         UNIT_MAINTENANCE,
         _UNIT_ACTIVE_STATE_MAX,
-        _UNIT_ACTIVE_STATE_INVALID = -1
+        _UNIT_ACTIVE_STATE_INVALID = -EINVAL,
 } UnitActiveState;
 
 typedef enum FreezerState {
@@ -54,16 +57,23 @@ typedef enum FreezerState {
         FREEZER_FROZEN,
         FREEZER_THAWING,
         _FREEZER_STATE_MAX,
-        _FREEZER_STATE_INVALID = -1
+        _FREEZER_STATE_INVALID = -EINVAL,
 } FreezerState;
 
+typedef enum UnitMarker {
+        UNIT_MARKER_NEEDS_RELOAD,
+        UNIT_MARKER_NEEDS_RESTART,
+        _UNIT_MARKER_MAX,
+        _UNIT_MARKER_INVALID = -EINVAL,
+} UnitMarker;
+
 typedef enum AutomountState {
         AUTOMOUNT_DEAD,
         AUTOMOUNT_WAITING,
         AUTOMOUNT_RUNNING,
         AUTOMOUNT_FAILED,
         _AUTOMOUNT_STATE_MAX,
-        _AUTOMOUNT_STATE_INVALID = -1
+        _AUTOMOUNT_STATE_INVALID = -EINVAL,
 } AutomountState;
 
 /* We simply watch devices, we cannot plug/unplug them. That
@@ -73,7 +83,7 @@ typedef enum DeviceState {
         DEVICE_TENTATIVE, /* mounted or swapped, but not (yet) announced by udev */
         DEVICE_PLUGGED,   /* announced by udev */
         _DEVICE_STATE_MAX,
-        _DEVICE_STATE_INVALID = -1
+        _DEVICE_STATE_INVALID = -EINVAL,
 } DeviceState;
 
 typedef enum MountState {
@@ -90,7 +100,7 @@ typedef enum MountState {
         MOUNT_FAILED,
         MOUNT_CLEANING,
         _MOUNT_STATE_MAX,
-        _MOUNT_STATE_INVALID = -1
+        _MOUNT_STATE_INVALID = -EINVAL,
 } MountState;
 
 typedef enum PathState {
@@ -99,18 +109,19 @@ typedef enum PathState {
         PATH_RUNNING,
         PATH_FAILED,
         _PATH_STATE_MAX,
-        _PATH_STATE_INVALID = -1
+        _PATH_STATE_INVALID = -EINVAL,
 } PathState;
 
 typedef enum ScopeState {
         SCOPE_DEAD,
+        SCOPE_START_CHOWN,
         SCOPE_RUNNING,
         SCOPE_ABANDONED,
         SCOPE_STOP_SIGTERM,
         SCOPE_STOP_SIGKILL,
         SCOPE_FAILED,
         _SCOPE_STATE_MAX,
-        _SCOPE_STATE_INVALID = -1
+        _SCOPE_STATE_INVALID = -EINVAL,
 } ScopeState;
 
 typedef enum ServiceState {
@@ -134,14 +145,14 @@ typedef enum ServiceState {
         SERVICE_AUTO_RESTART,
         SERVICE_CLEANING,
         _SERVICE_STATE_MAX,
-        _SERVICE_STATE_INVALID = -1
+        _SERVICE_STATE_INVALID = -EINVAL,
 } ServiceState;
 
 typedef enum SliceState {
         SLICE_DEAD,
         SLICE_ACTIVE,
         _SLICE_STATE_MAX,
-        _SLICE_STATE_INVALID = -1
+        _SLICE_STATE_INVALID = -EINVAL,
 } SliceState;
 
 typedef enum SocketState {
@@ -160,7 +171,7 @@ typedef enum SocketState {
         SOCKET_FAILED,
         SOCKET_CLEANING,
         _SOCKET_STATE_MAX,
-        _SOCKET_STATE_INVALID = -1
+        _SOCKET_STATE_INVALID = -EINVAL,
 } SocketState;
 
 typedef enum SwapState {
@@ -174,14 +185,14 @@ typedef enum SwapState {
         SWAP_FAILED,
         SWAP_CLEANING,
         _SWAP_STATE_MAX,
-        _SWAP_STATE_INVALID = -1
+        _SWAP_STATE_INVALID = -EINVAL,
 } SwapState;
 
 typedef enum TargetState {
         TARGET_DEAD,
         TARGET_ACTIVE,
         _TARGET_STATE_MAX,
-        _TARGET_STATE_INVALID = -1
+        _TARGET_STATE_INVALID = -EINVAL,
 } TargetState;
 
 typedef enum TimerState {
@@ -191,7 +202,7 @@ typedef enum TimerState {
         TIMER_ELAPSED,
         TIMER_FAILED,
         _TIMER_STATE_MAX,
-        _TIMER_STATE_INVALID = -1
+        _TIMER_STATE_INVALID = -EINVAL,
 } TimerState;
 
 typedef enum UnitDependency {
@@ -201,6 +212,7 @@ typedef enum UnitDependency {
         UNIT_WANTS,
         UNIT_BINDS_TO,
         UNIT_PART_OF,
+        UNIT_UPHOLDS,
 
         /* Inverse of the above */
         UNIT_REQUIRED_BY,             /* inverse of 'requires' is 'required_by' */
@@ -208,6 +220,7 @@ typedef enum UnitDependency {
         UNIT_WANTED_BY,               /* inverse of 'wants' */
         UNIT_BOUND_BY,                /* inverse of 'binds_to' */
         UNIT_CONSISTS_OF,             /* inverse of 'part_of' */
+        UNIT_UPHELD_BY,               /* inverse of 'uphold' */
 
         /* Negative dependencies */
         UNIT_CONFLICTS,               /* inverse of 'conflicts' is 'conflicted_by' */
@@ -217,8 +230,11 @@ typedef enum UnitDependency {
         UNIT_BEFORE,                  /* inverse of 'before' is 'after' and vice versa */
         UNIT_AFTER,
 
-        /* On Failure */
+        /* OnSuccess= + OnFailure= */
+        UNIT_ON_SUCCESS,
+        UNIT_ON_SUCCESS_OF,
         UNIT_ON_FAILURE,
+        UNIT_ON_FAILURE_OF,
 
         /* Triggers (i.e. a socket triggers a service) */
         UNIT_TRIGGERS,
@@ -228,6 +244,10 @@ typedef enum UnitDependency {
         UNIT_PROPAGATES_RELOAD_TO,
         UNIT_RELOAD_PROPAGATED_FROM,
 
+        /* Propagate stops */
+        UNIT_PROPAGATES_STOP_TO,
+        UNIT_STOP_PROPAGATED_FROM,
+
         /* Joins namespace of */
         UNIT_JOINS_NAMESPACE_OF,
 
@@ -235,8 +255,12 @@ typedef enum UnitDependency {
         UNIT_REFERENCES,              /* Inverse of 'references' is 'referenced_by' */
         UNIT_REFERENCED_BY,
 
+        /* Slice= */
+        UNIT_IN_SLICE,
+        UNIT_SLICE_OF,
+
         _UNIT_DEPENDENCY_MAX,
-        _UNIT_DEPENDENCY_INVALID = -1
+        _UNIT_DEPENDENCY_INVALID = -EINVAL,
 } UnitDependency;
 
 typedef enum NotifyAccess {
@@ -245,11 +269,11 @@ typedef enum NotifyAccess {
         NOTIFY_MAIN,
         NOTIFY_EXEC,
         _NOTIFY_ACCESS_MAX,
-        _NOTIFY_ACCESS_INVALID = -1
+        _NOTIFY_ACCESS_INVALID = -EINVAL,
 } NotifyAccess;
 
-char *unit_dbus_path_from_name(const char *name);
 #if 0 /// elogind does not support systemd units
+char *unit_dbus_path_from_name(const char *name);
 int unit_name_from_dbus_path(const char *path, char **name);
 
 const char* unit_dbus_interface_from_type(UnitType t);
@@ -269,6 +293,9 @@ UnitActiveState unit_active_state_from_string(const char *s) _pure_;
 const char *freezer_state_to_string(FreezerState i) _const_;
 FreezerState freezer_state_from_string(const char *s) _pure_;
 
+const char *unit_marker_to_string(UnitMarker m) _const_;
+UnitMarker unit_marker_from_string(const char *s) _pure_;
+
 const char* automount_state_to_string(AutomountState i) _const_;
 AutomountState automount_state_from_string(const char *s) _pure_;
 
@@ -307,4 +334,6 @@ UnitDependency unit_dependency_from_string(const char *s) _pure_;
 
 const char* notify_access_to_string(NotifyAccess i) _const_;
 NotifyAccess notify_access_from_string(const char *s) _pure_;
+
+SpecialGlyph unit_active_state_to_glyph(UnitActiveState state);
 #endif // 0
