@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "sd-path.h"
 
@@ -12,7 +12,7 @@
 #include "string-util.h"
 #include "strv.h"
 #include "user-util.h"
-#include "util.h"
+//#include "util.h"
 
 static int from_environment(const char *envname, const char *fallback, const char **ret) {
         assert(ret);
@@ -37,7 +37,6 @@ static int from_environment(const char *envname, const char *fallback, const cha
 
 static int from_home_dir(const char *envname, const char *suffix, char **buffer, const char **ret) {
         _cleanup_free_ char *h = NULL;
-        char *cc = NULL;
         int r;
 
         assert(suffix);
@@ -58,12 +57,11 @@ static int from_home_dir(const char *envname, const char *suffix, char **buffer,
         if (r < 0)
                 return r;
 
-        cc = path_join(h, suffix);
-        if (!cc)
+        if (!path_extend(&h, suffix))
                 return -ENOMEM;
 
-        *buffer = cc;
-        *ret = cc;
+        *buffer = h;
+        *ret = TAKE_PTR(h);
         return 0;
 }
 
@@ -135,18 +133,16 @@ static int from_user_dir(const char *field, char **buffer, const char **ret) {
                 /* Three syntaxes permitted: relative to $HOME, $HOME itself, and absolute path */
                 if (startswith(p, "$HOME/")) {
                         _cleanup_free_ char *h = NULL;
-                        char *cc;
 
                         r = get_home_dir(&h);
                         if (r < 0)
                                 return r;
 
-                        cc = path_join(h, p+5);
-                        if (!cc)
+                        if (!path_extend(&h, p+5))
                                 return -ENOMEM;
 
-                        *buffer = cc;
-                        *ret = cc;
+                        *buffer = h;
+                        *ret = TAKE_PTR(h);
                         return 0;
                 } else if (streq(p, "$HOME")) {
 
@@ -173,20 +169,17 @@ fallback:
         /* The desktop directory defaults to $HOME/Desktop, the others to $HOME */
         if (streq(field, "XDG_DESKTOP_DIR")) {
                 _cleanup_free_ char *h = NULL;
-                char *cc;
 
                 r = get_home_dir(&h);
                 if (r < 0)
                         return r;
 
-                cc = path_join(h, "Desktop");
-                if (!cc)
+                if (!path_extend(&h, "Desktop"))
                         return -ENOMEM;
 
-                *buffer = cc;
-                *ret = cc;
+                *buffer = h;
+                *ret = TAKE_PTR(h);
         } else {
-
                 r = get_home_dir(buffer);
                 if (r < 0)
                         return r;
@@ -330,7 +323,7 @@ static int get_path(uint64_t type, char **buffer, const char **ret) {
 
 #if 0 /// UNNEEDED by elogind
         case SD_PATH_SYSTEMD_SYSTEM_UNIT:
-                *ret = SYSTEM_DATA_UNIT_PATH;
+                *ret = SYSTEM_DATA_UNIT_DIR;
                 return 0;
 
         case SD_PATH_SYSTEMD_SYSTEM_PRESET:
@@ -378,8 +371,7 @@ static int get_path(uint64_t type, char **buffer, const char **ret) {
 #endif // 0
                 return 0;
 
-        /* FIXME: systemd.pc uses ${prefix}, but CONF_PATHS_NULSTR doesn't.
-         *        Should ${prefix} use in systemd.pc be removed? */
+#if 0 /// UNNEEDED by elogind
         case SD_PATH_TMPFILES:
                 *ret = "/usr/lib/tmpfiles.d";
                 return 0;
@@ -403,6 +395,7 @@ static int get_path(uint64_t type, char **buffer, const char **ret) {
         case SD_PATH_CATALOG:
                 *ret = "/usr/lib/systemd/catalog";
                 return 0;
+#endif // 0
         }
 
         return -EOPNOTSUPP;
@@ -541,7 +534,7 @@ static int get_search(uint64_t type, char ***list) {
 
         assert(list);
 
-        switch(type) {
+        switch (type) {
 
         case SD_PATH_SEARCH_BINARIES:
                 return search_from_environment(list,
@@ -627,8 +620,8 @@ static int get_search(uint64_t type, char ***list) {
         case SD_PATH_SYSTEMD_SEARCH_SYSTEM_UNIT:
         case SD_PATH_SYSTEMD_SEARCH_USER_UNIT: {
                 _cleanup_(lookup_paths_free) LookupPaths lp = {};
-                const UnitFileScope scope = type == SD_PATH_SYSTEMD_SEARCH_SYSTEM_UNIT ?
-                                                    UNIT_FILE_SYSTEM : UNIT_FILE_USER;
+                const LookupScope scope = type == SD_PATH_SYSTEMD_SEARCH_SYSTEM_UNIT ?
+                                                    LOOKUP_SCOPE_SYSTEM : LOOKUP_SCOPE_USER;
 
                 r = lookup_paths_init(&lp, scope, 0, NULL);
                 if (r < 0)
@@ -641,8 +634,8 @@ static int get_search(uint64_t type, char ***list) {
         case SD_PATH_SYSTEMD_SEARCH_SYSTEM_GENERATOR:
         case SD_PATH_SYSTEMD_SEARCH_USER_GENERATOR: {
                 char **t;
-                const UnitFileScope scope = type == SD_PATH_SYSTEMD_SEARCH_SYSTEM_GENERATOR ?
-                                                    UNIT_FILE_SYSTEM : UNIT_FILE_USER;
+                const LookupScope scope = type == SD_PATH_SYSTEMD_SEARCH_SYSTEM_GENERATOR ?
+                                                    LOOKUP_SCOPE_SYSTEM : LOOKUP_SCOPE_USER;
 
                 t = generator_binary_paths(scope);
                 if (!t)
@@ -696,7 +689,7 @@ _public_ int sd_path_lookup_strv(uint64_t type, const char *suffix, char ***path
         if (!n)
                 return -ENOMEM;
 
-        char **i, **j = n;
+        char **j = n;
         STRV_FOREACH(i, l) {
                 *j = path_join(*i, suffix);
                 if (!*j)
