@@ -1,7 +1,7 @@
 /***
   This file is part of elogind.
 
-  Copyright 2017-2018 Sven Eden
+  Copyright 2017 Sven Eden
 
   elogind is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published by
@@ -38,7 +38,6 @@
 elogind_action arg_action            = _ACTION_INVALID;
 bool           arg_ask_password      = true;
 bool           arg_dry_run           = false;
-bool           arg_quiet             = false;
 bool           arg_ignore_inhibitors = false;
 bool           arg_no_wall           = false;
 BusTransport   arg_transport         = BUS_TRANSPORT_LOCAL;
@@ -84,7 +83,6 @@ static int check_inhibitors(sd_bus* bus, enum elogind_action a) {
         const char *what, *who, *why, *mode;
         uint32_t uid, pid;
         unsigned c = 0;
-        char **s;
         int r;
 
         if (arg_ignore_inhibitors)
@@ -377,7 +375,6 @@ static int elogind_reboot(sd_bus *bus, enum elogind_action a) {
  */
 static int elogind_schedule_shutdown(sd_bus *bus, enum elogind_action a) {
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
-        char date[FORMAT_TIMESTAMP_MAX];
         const char *action;
         int r;
 
@@ -422,9 +419,6 @@ static int elogind_schedule_shutdown(sd_bus *bus, enum elogind_action a) {
                                 "Failed to call ScheduleShutdown in logind, proceeding with immediate shutdown: %s",
                                 bus_error_message(&error, r));
 
-        if (!arg_quiet)
-                log_info("Shutdown scheduled for %s, use 'shutdown -c' to cancel.", format_timestamp(date, sizeof(date), arg_when));
-
         return 0;
 }
 
@@ -575,7 +569,11 @@ static int prepare_firmware_setup(sd_bus* bus) {
 
         return 0;
 }
-#endif
+#else // 0
+static int prepare_firmware_setup(sd_bus* bus) {
+        return 0;
+}
+#endif // ENABLE_EFI
 
 #ifdef ENABLE_EFI_TODO /// @todo EFI - needs change to support UEFI boot.
 /* Original:
@@ -637,7 +635,14 @@ static int prepare_boot_loader_entry(sd_bus* bus) {
 
         return 0;
 }
-#endif // ENABLE_EFI
+#else // 0
+static int prepare_boot_loader_menu(sd_bus* bus) {
+        return 0;
+}
+static int prepare_boot_loader_entry(sd_bus* bus) {
+        return 0;
+}
+#endif // ENABLE_EFI_TODO
 
 /* Original:
  * systemctl/systemctl.c:3482:start_special()
@@ -705,8 +710,6 @@ int start_special(int argc, char *argv[], void *userdata) {
         if (r < 0)
                 return r;
 
-#if ENABLE_EFI
-#ifdef ENABLE_EFI_TODO /// @todo EFI - needs change to support UEFI boot.
         r = prepare_boot_loader_menu(bus);
         if (r < 0)
                 return r;
@@ -716,8 +719,6 @@ int start_special(int argc, char *argv[], void *userdata) {
                 return r;
         if (r > 0)
                 return 0; // Asked for help, no execution, then
-#endif // ENABLE_EFI_TODO
-#endif // ENABLE_EFI
 
         if (a == ACTION_REBOOT && argc > 1) {
                 r = update_reboot_parameter_and_warn(argv[1], false);
