@@ -1,7 +1,7 @@
 /***
   This file is part of elogind.
 
-  Copyright 2017-2018 Sven Eden
+  Copyright 2017 Sven Eden
 
   elogind is free software; you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as published by
@@ -38,16 +38,13 @@
 
 #include <errno.h>
 
-/// defined in sd-bus.c
-void bus_reset_queues( sd_bus* b );
-
 #define CGROUPS_AGENT_RCVBUF_SIZE (8*1024*1024)
 #ifndef ELOGIND_PID_FILE
 #  define ELOGIND_PID_FILE "/run/elogind.pid"
 #endif // ELOGIND_PID_FILE
 
 
-/* The elogind specific signal handler sends an exit event, so eleogind can
+/* The elogind specific signal handler sends an exit event, so elogind can
    gracefully shutdown.
    Caught are SIGINT, SIGQUIT and SIGTERM.
    While QUIT and TERMinate mean to power down the service completely, INTerrupt
@@ -64,9 +61,7 @@ static int elogind_signal_handler(
         Manager* m = userdata;
         int r;
 
-        log_warning( "Received signal %u [%s]", si->ssi_signo,
-                     signal_to_string( si->ssi_signo )
-                   );
+        log_warning( "Received signal %u [%s]", si->ssi_signo, signal_to_string( si->ssi_signo ) );
 
         r = sd_event_get_state( m->event );
 
@@ -249,8 +244,7 @@ int elogind_setup_cgroups_agent( Manager* m ) {
                         .un.sun_family = AF_UNIX,
                         .un.sun_path = "/run/systemd/cgroups-agent",
         };
-
-        int r;
+        int                               r  = 0;
 
         /* This creates a listening socket we receive cgroups agent messages on. We do not use D-Bus for delivering
          * these messages from the cgroups agent binary to PID 1, as the cgroups agent binary is very short-living, and
@@ -278,7 +272,7 @@ int elogind_setup_cgroups_agent( Manager* m ) {
                 return 0;
 
         if ( m->cgroups_agent_fd < 0 ) {
-                _cleanup_close_ int fd;
+                _cleanup_close_ int fd = -1;
 
                 /* First free all secondary fields */
                 m->cgroups_agent_event_source = sd_event_source_unref( m->cgroups_agent_event_source );
@@ -287,7 +281,7 @@ int elogind_setup_cgroups_agent( Manager* m ) {
                 if ( fd < 0 )
                         return log_error_errno( errno, "Failed to allocate cgroups agent socket: %m" );
 
-                fd_inc_rcvbuf( fd, CGROUPS_AGENT_RCVBUF_SIZE );
+                fd_increase_rxbuf( fd, CGROUPS_AGENT_RCVBUF_SIZE );
 
                 (void) unlink( sa.un.sun_path );
 
@@ -331,9 +325,9 @@ int elogind_startup(int argc, char *argv[], bool *has_forked) {
         int i, r;
         pid_t pid;
 
-        /* if elogind was called with -h/--help, skip all checks */
-        for (i = 1; (i <= argc) && argv[i]; ++i ) {
-                if (streq("-h", argv[i]) || streq("--help", argv[i]))
+        /* if elogind was called with -h/--help or --bus-introspect, skip all checks */
+        for (i = 1; !daemonize && (i <= argc) && argv[i]; ++i ) {
+                if (streq("-h", argv[i]) || streq("--help", argv[i]) || startswith(argv[i], "--bus-introspect") )
                         return 0;
         }
 
@@ -375,12 +369,10 @@ void elogind_manager_free( Manager* m ) {
 
         safe_close( m->cgroups_agent_fd );
 
-        strv_free( m->suspend_modes );
-        strv_free( m->suspend_states );
-        strv_free( m->hibernate_modes );
-        strv_free( m->hibernate_states );
-        strv_free( m->hybrid_modes );
-        strv_free( m->hybrid_states );
+        for (SleepOperation i = 0; i < _SLEEP_OPERATION_MAX; i++) {
+                strv_free(m->modes[i]);
+                strv_free(m->states[i]);
+        }
 }
 
 
@@ -404,18 +396,13 @@ int elogind_manager_new( Manager* m ) {
         /* allow manipulating Nvidia cards */
         m->handle_nvidia_sleep = false;
 
-        /* Init sleep modes and states */
-        m->suspend_modes       = NULL;
-        m->suspend_states      = NULL;
-        m->hibernate_modes     = NULL;
-        m->hibernate_states    = NULL;
-        m->hybrid_modes        = NULL;
-        m->hybrid_states       = NULL;
-        m->hibernate_delay_sec = 0;
-        m->allow_suspend       = true;
-        m->allow_hibernate     = true;
-        m->allow_hybrid_sleep  = true;
-        m->allow_s2h           = true;
+        /* Init sleep values */
+        m->allow[SLEEP_SUSPEND]                = true;
+        m->allow[SLEEP_HIBERNATE]              = true;
+        m->allow[SLEEP_HYBRID_SLEEP]           = true;
+        m->allow[SLEEP_SUSPEND_THEN_HIBERNATE] = true;
+        m->hibernate_delay_usec                = 0;
+        m->suspend_estimation_usec             = 0;
 
         /* If elogind should be its own controller, mount its cgroup */
         if ( streq( SYSTEMD_CGROUP_CONTROLLER, "_elogind" ) ) {
@@ -439,7 +426,7 @@ int elogind_manager_startup( Manager* m ) {
         /* Install our signal handler */
         r = sd_event_add_signal( m->event, NULL, SIGINT, elogind_signal_handler, m );
         if ( r < 0 ) {
-                e = r;
+                if ( e == 0 ) e = r;
                 log_error_errno( r, "Failed to register SIGINT handler: %m" );
         }
 
