@@ -1,34 +1,33 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <fcntl.h>
-#include <sys/ioctl.h>
+//#include <sys/ioctl.h>
 #include <sys/types.h>
-#include <linux/vt.h>
+//#include <linux/vt.h>
 
 #include "sd-device.h"
 
 #include "alloc-util.h"
-#include "bus-error.h"
-#include "bus-util.h"
+//#include "bus-error.h"
+//#include "bus-util.h"
 #include "cgroup-util.h"
 #include "conf-parser.h"
 #include "device-util.h"
 #include "efi-loader.h"
 #include "errno-util.h"
-#include "fd-util.h"
+//#include "fd-util.h"
 #include "limits-util.h"
 #include "logind.h"
-#include "parse-util.h"
+//#include "parse-util.h"
 #include "path-util.h"
 #include "process-util.h"
+#include "stdio-util.h"
 #include "strv.h"
-#include "terminal-util.h"
+//#include "terminal-util.h"
 #include "udev-util.h"
 #include "user-util.h"
 #include "userdb.h"
 /// Additional includes needed by elogind
-#include "elogind.h"
-#include "sleep-config.h"
 #include "utmp-wtmp.h"
 
 void manager_reset_config(Manager *m) {
@@ -47,15 +46,23 @@ void manager_reset_config(Manager *m) {
         m->user_stop_delay = 0;
 #endif // 0
         m->handle_power_key = HANDLE_POWEROFF;
+        m->handle_power_key_long_press = HANDLE_IGNORE;
+        m->handle_reboot_key = HANDLE_REBOOT;
+        m->handle_reboot_key_long_press = HANDLE_POWEROFF;
         m->handle_suspend_key = HANDLE_SUSPEND;
+        m->handle_suspend_key_long_press = HANDLE_HIBERNATE;
         m->handle_hibernate_key = HANDLE_HIBERNATE;
+        m->handle_hibernate_key_long_press = HANDLE_IGNORE;
+
         m->handle_lid_switch = HANDLE_SUSPEND;
         m->handle_lid_switch_ep = _HANDLE_ACTION_INVALID;
         m->handle_lid_switch_docked = HANDLE_IGNORE;
+
         m->power_key_ignore_inhibited = false;
         m->suspend_key_ignore_inhibited = false;
         m->hibernate_key_ignore_inhibited = false;
         m->lid_switch_ignore_inhibited = true;
+        m->reboot_key_ignore_inhibited = false;
 
         m->holdoff_timeout_usec = 30 * USEC_PER_SEC;
 
@@ -71,49 +78,10 @@ void manager_reset_config(Manager *m) {
 
         m->kill_only_users = strv_free(m->kill_only_users);
         m->kill_exclude_users = strv_free(m->kill_exclude_users);
+
+        m->stop_idle_session_usec = USEC_INFINITY;
 }
 
-#if 1 /// Add-On for manager_reset_config() used by elogind
-void elogind_manager_reset_config(Manager* m) {
-
-#if ENABLE_DEBUG_ELOGIND
-        int dbg_cnt;
-#endif // ENABLE_DEBUG_ELOGIND
-
-        (void)parse_sleep_config(&m);
-
-#if ENABLE_DEBUG_ELOGIND
-        dbg_cnt = -1;
-        while (m->suspend_modes && m->suspend_modes[++dbg_cnt])
-                log_debug_elogind("suspend_modes[%d] = %s",
-                                  dbg_cnt, m->suspend_modes[dbg_cnt]);
-        dbg_cnt = -1;
-        while (m->suspend_states && m->suspend_states[++dbg_cnt])
-                log_debug_elogind("suspend_states[%d] = %s",
-                                  dbg_cnt, m->suspend_states[dbg_cnt]);
-        dbg_cnt = -1;
-        while (m->hibernate_modes && m->hibernate_modes[++dbg_cnt])
-                log_debug_elogind("hibernate_modes[%d] = %s",
-                                  dbg_cnt, m->hibernate_modes[dbg_cnt]);
-        dbg_cnt = -1;
-        while (m->hibernate_states && m->hibernate_states[++dbg_cnt])
-                log_debug_elogind("hibernate_states[%d] = %s",
-                                  dbg_cnt, m->hibernate_states[dbg_cnt]);
-        dbg_cnt = -1;
-        while (m->hybrid_modes && m->hybrid_modes[++dbg_cnt])
-                log_debug_elogind("hybrid_modes[%d] = %s",
-                                  dbg_cnt, m->hybrid_modes[dbg_cnt]);
-        dbg_cnt = -1;
-        while (m->hybrid_states && m->hybrid_states[++dbg_cnt])
-                log_debug_elogind("hybrid_states[%d] = %s",
-                                  dbg_cnt, m->hybrid_states[dbg_cnt]);
-        log_debug_elogind("hibernate_delay_sec: %lu seconds (%lu minutes)",
-                          m->hibernate_delay_sec / USEC_PER_SEC,
-                          m->hibernate_delay_sec / USEC_PER_MINUTE);
-#endif // ENABLE_DEBUG_ELOGIND
-}
-#endif // 1
-
 int manager_parse_config_file(Manager *m) {
         assert(m);
 
@@ -238,7 +206,7 @@ int manager_add_user_by_name(
         assert(m);
         assert(name);
 
-        r = userdb_by_name(name, USERDB_AVOID_SHADOW, &ur);
+        r = userdb_by_name(name, USERDB_SUPPRESS_SHADOW, &ur);
         if (r < 0)
                 return r;
 
@@ -256,7 +224,7 @@ int manager_add_user_by_uid(
         assert(m);
         assert(uid_is_valid(uid));
 
-        r = userdb_by_uid(uid, USERDB_AVOID_SHADOW, &ur);
+        r = userdb_by_uid(uid, USERDB_SUPPRESS_SHADOW, &ur);
         if (r < 0)
                 return r;
 
@@ -308,7 +276,8 @@ int manager_process_seat_device(Manager *m, sd_device *d) {
 
         assert(m);
 
-        if (device_for_action(d, DEVICE_ACTION_REMOVE)) {
+        if (device_for_action(d, SD_DEVICE_REMOVE) ||
+            sd_device_has_current_tag(d, "seat") <= 0) {
                 const char *syspath;
 
                 r = sd_device_get_syspath(d, &syspath);
@@ -336,7 +305,7 @@ int manager_process_seat_device(Manager *m, sd_device *d) {
                 }
 
                 seat = hashmap_get(m->seats, sn);
-                master = sd_device_has_tag(d, "master-of-seat") > 0;
+                master = sd_device_has_current_tag(d, "master-of-seat") > 0;
 
                 /* Ignore non-master devices for unknown seats */
                 if (!master && !seat)
@@ -378,7 +347,8 @@ int manager_process_button_device(Manager *m, sd_device *d) {
         if (r < 0)
                 return r;
 
-        if (device_for_action(d, DEVICE_ACTION_REMOVE)) {
+        if (device_for_action(d, SD_DEVICE_REMOVE) ||
+            sd_device_has_current_tag(d, "power-switch") <= 0) {
 
                 b = hashmap_get(m->buttons, sysname);
                 if (!b)
@@ -428,7 +398,7 @@ int manager_get_session_by_pid(Manager *m, pid_t pid, Session **ret) {
                 if (r >= 0)
                         s = hashmap_get(m->session_units, unit);
 #else // 0
-                log_debug_elogind("Searching session for PID %u", pid);
+                log_debug_elogind("Searching session for PID %d", pid);
                 r = cg_pid_get_session(pid, &session_name);
 
                 if (r >= 0)
@@ -446,11 +416,10 @@ int manager_get_session_by_pid(Manager *m, pid_t pid, Session **ret) {
 }
 
 int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
-#if 0 /// elogind does not support systemd units, but its own session system
         _cleanup_free_ char *unit = NULL;
-#else // 0
+#if 1 /// elogind has its own session system
         Session *s;
-#endif // 0
+#endif // 1
         User *u = NULL;
         int r;
 
@@ -459,16 +428,18 @@ int manager_get_user_by_pid(Manager *m, pid_t pid, User **ret) {
         if (!pid_is_valid(pid))
                 return -EINVAL;
 
-#if 0 /// elogind does not support systemd units, but its own session system
         r = cg_pid_get_slice(pid, &unit);
         if (r >= 0)
                 u = hashmap_get(m->user_units, unit);
-#else // 0
-        // If a session was found, ignore it if it is already closing.
-        r = manager_get_session_by_pid (m, pid, &s);
-        if ( (r > 0) && (SESSION_CLOSING != session_get_state(s)) )
-                u = s->user;
-#endif // 0
+
+#if 1 /// elogind also has its own session system
+        if ((r < 0) || (NULL == u)) {
+                r = manager_get_session_by_pid(m, pid, &s);
+                // If a session was found, ignore it if it is already closing.
+                if ((r > 0) && (SESSION_CLOSING != session_get_state(s)))
+                        u = s->user;
+        }
+#endif // 1
 
         if (ret)
                 *ret = u;
@@ -480,13 +451,12 @@ int manager_get_idle_hint(Manager *m, dual_timestamp *t) {
         Session *s;
         bool idle_hint;
         dual_timestamp ts = DUAL_TIMESTAMP_NULL;
-        Iterator i;
 
         assert(m);
 
         idle_hint = !manager_is_inhibited(m, INHIBIT_IDLE, INHIBIT_BLOCK, t, false, false, 0, NULL);
 
-        HASHMAP_FOREACH(s, m->sessions, i) {
+        HASHMAP_FOREACH(s, m->sessions) {
                 dual_timestamp k;
                 int ih;
 
@@ -544,23 +514,24 @@ int config_parse_n_autovts(
                 void *data,
                 void *userdata) {
 
-        unsigned *n = data;
+        unsigned *n = ASSERT_PTR(data);
         unsigned o;
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
 
         r = safe_atou(rvalue, &o);
         if (r < 0) {
-                log_syntax(unit, LOG_ERR, filename, line, r, "Failed to parse number of autovts, ignoring: %s", rvalue);
+                log_syntax(unit, LOG_WARNING, filename, line, r,
+                           "Failed to parse number of autovts, ignoring: %s", rvalue);
                 return 0;
         }
 
         if (o > 15) {
-                log_syntax(unit, LOG_ERR, filename, line, r, "A maximum of 15 autovts are supported, ignoring: %s", rvalue);
+                log_syntax(unit, LOG_WARNING, filename, line, 0,
+                           "A maximum of 15 autovts are supported, ignoring: %s", rvalue);
                 return 0;
         }
 
@@ -570,8 +541,8 @@ int config_parse_n_autovts(
 
 static int vt_is_busy(unsigned vtnr) {
         struct vt_stat vt_stat;
-        int r = 0;
-        _cleanup_close_ int fd;
+        int r;
+        _cleanup_close_ int fd = -1;
 
         assert(vtnr >= 1);
 
@@ -620,7 +591,7 @@ int manager_spawn_autovt(Manager *m, unsigned vtnr) {
                         return -EBUSY;
         }
 
-        snprintf(name, sizeof(name), "autovt@tty%u.service", vtnr);
+        xsprintf(name, "autovt@tty%u.service", vtnr);
         r = sd_bus_call_method(
                         m->bus,
                         "org.freedesktop.systemd1",
@@ -638,10 +609,9 @@ int manager_spawn_autovt(Manager *m, unsigned vtnr) {
 #endif // 0
 
 bool manager_is_lid_closed(Manager *m) {
-        Iterator i;
         Button *b;
 
-        HASHMAP_FOREACH(b, m->buttons, i)
+        HASHMAP_FOREACH(b, m->buttons)
                 if (b->lid_closed)
                         return true;
 
@@ -649,10 +619,9 @@ bool manager_is_lid_closed(Manager *m) {
 }
 
 static bool manager_is_docked(Manager *m) {
-        Iterator i;
         Button *b;
 
-        HASHMAP_FOREACH(b, m->buttons, i)
+        HASHMAP_FOREACH(b, m->buttons)
                 if (b->docked)
                         return true;
 
@@ -759,10 +728,20 @@ bool manager_all_buttons_ignored(Manager *m) {
 
         if (m->handle_power_key != HANDLE_IGNORE)
                 return false;
+        if (m->handle_power_key_long_press != HANDLE_IGNORE)
+                return false;
         if (m->handle_suspend_key != HANDLE_IGNORE)
                 return false;
+        if (m->handle_suspend_key_long_press != HANDLE_IGNORE)
+                return false;
         if (m->handle_hibernate_key != HANDLE_IGNORE)
                 return false;
+        if (m->handle_hibernate_key_long_press != HANDLE_IGNORE)
+                return false;
+        if (m->handle_reboot_key != HANDLE_IGNORE)
+                return false;
+        if (m->handle_reboot_key_long_press != HANDLE_IGNORE)
+                return false;
         if (m->handle_lid_switch != HANDLE_IGNORE)
                 return false;
         if (!IN_SET(m->handle_lid_switch_ep, _HANDLE_ACTION_INVALID, HANDLE_IGNORE))
@@ -776,7 +755,7 @@ bool manager_all_buttons_ignored(Manager *m) {
 int manager_read_utmp(Manager *m) {
 #if ENABLE_UTMP
         int r;
-        _cleanup_(utxent_cleanup) bool utmpx = false;
+        _unused_ _cleanup_(utxent_cleanup) bool utmpx = false;
 
         assert(m);
 
@@ -794,7 +773,9 @@ int manager_read_utmp(Manager *m) {
                 errno = 0;
                 u = getutxent();
                 if (!u) {
-                        if (errno != 0)
+                        if (errno == ENOENT)
+                                log_debug_errno(errno, _PATH_UTMPX " does not exist, ignoring.");
+                        else if (errno != 0)
                                 log_warning_errno(errno, "Failed to read " _PATH_UTMPX ", ignoring: %m");
                         return 0;
                 }
@@ -850,9 +831,7 @@ int manager_read_utmp(Manager *m) {
 
 #if ENABLE_UTMP
 static int manager_dispatch_utmp(sd_event_source *s, const struct inotify_event *event, void *userdata) {
-        Manager *m = userdata;
-
-        assert(m);
+        Manager *m = ASSERT_PTR(userdata);
 
         /* If there's indication the file itself might have been removed or became otherwise unavailable, then let's
          * reestablish the watch on whatever there's now. */
