@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
 
@@ -8,9 +8,11 @@
 #include "bus-label.h"
 #include "bus-polkit.h"
 #include "bus-util.h"
+#include "devnum-util.h"
 #include "fd-util.h"
 #include "logind-brightness.h"
 #include "logind-dbus.h"
+#include "logind-polkit.h"
 #include "logind-seat-dbus.h"
 #include "logind-session-dbus.h"
 #include "logind-session-device.h"
@@ -20,7 +22,6 @@
 #include "missing_capability.h"
 #include "path-util.h"
 #include "signal-util.h"
-#include "stat-util.h"
 #include "strv.h"
 #include "user-util.h"
 #include "util.h"
@@ -35,11 +36,10 @@ static int property_get_user(
                 sd_bus_error *error) {
 
         _cleanup_free_ char *p = NULL;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         p = user_bus_path(s->user);
         if (!p)
@@ -57,11 +57,10 @@ static int property_get_name(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         return sd_bus_message_append(reply, "s", s->user->user_record->user_name);
 }
@@ -76,11 +75,10 @@ static int property_get_seat(
                 sd_bus_error *error) {
 
         _cleanup_free_ char *p = NULL;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         p = s->seat ? seat_bus_path(s->seat) : strdup("/");
         if (!p)
@@ -103,11 +101,10 @@ static int property_get_idle_hint(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         return sd_bus_message_append(reply, "b", session_get_idle_hint(s, NULL) > 0);
 }
@@ -121,14 +118,13 @@ static int property_get_idle_since_hint(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         dual_timestamp t = DUAL_TIMESTAMP_NULL;
         uint64_t u;
         int r;
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         r = session_get_idle_hint(s, &t);
         if (r < 0)
@@ -148,21 +144,19 @@ static int property_get_locked_hint(
                 void *userdata,
                 sd_bus_error *error) {
 
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(bus);
         assert(reply);
-        assert(s);
 
         return sd_bus_message_append(reply, "b", session_get_locked_hint(s) > 0);
 }
 
 int bus_session_method_terminate(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         int r;
 
         assert(message);
-        assert(s);
 
         r = bus_verify_polkit_async(
                         message,
@@ -180,7 +174,7 @@ int bus_session_method_terminate(sd_bus_message *message, void *userdata, sd_bus
 
         log_debug_elogind("UID %u called dbus method 'terminate' on session %s",
                           s->user->user_record->uid, s->id);
-        r = session_stop(s, true);
+        r = session_stop(s, /* force = */ true);
         if (r < 0)
                 return r;
 
@@ -188,21 +182,12 @@ int bus_session_method_terminate(sd_bus_message *message, void *userdata, sd_bus
 }
 
 int bus_session_method_activate(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         int r;
 
         assert(message);
-        assert(s);
 
-        r = bus_verify_polkit_async(
-                        message,
-                        CAP_SYS_ADMIN,
-                        "org.freedesktop.login1.chvt",
-                        NULL,
-                        false,
-                        UID_INVALID,
-                        &s->manager->polkit_registry,
-                        error);
+        r = check_polkit_chvt(message, s->manager, error);
         if (r < 0)
                 return r;
         if (r == 0)
@@ -216,11 +201,10 @@ int bus_session_method_activate(sd_bus_message *message, void *userdata, sd_bus_
 }
 
 int bus_session_method_lock(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         int r;
 
         assert(message);
-        assert(s);
 
         r = bus_verify_polkit_async(
                         message,
@@ -245,12 +229,11 @@ int bus_session_method_lock(sd_bus_message *message, void *userdata, sd_bus_erro
 
 static int method_set_idle_hint(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(sd_bus_creds_unrefp) sd_bus_creds *creds = NULL;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uid_t uid;
         int r, b;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "b", &b);
         if (r < 0)
@@ -265,11 +248,11 @@ static int method_set_idle_hint(sd_bus_message *message, void *userdata, sd_bus_
                 return r;
 
         if (uid != 0 && uid != s->user->user_record->uid)
-                return sd_bus_error_setf(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may set idle hint");
+                return sd_bus_error_set(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may set idle hint");
 
         r = session_set_idle_hint(s, b);
         if (r == -ENOTTY)
-                return sd_bus_error_setf(error, SD_BUS_ERROR_NOT_SUPPORTED, "Idle hint control is not supported on non-graphical sessions.");
+                return sd_bus_error_set(error, SD_BUS_ERROR_NOT_SUPPORTED, "Idle hint control is not supported on non-graphical sessions.");
         if (r < 0)
                 return r;
 
@@ -278,12 +261,11 @@ static int method_set_idle_hint(sd_bus_message *message, void *userdata, sd_bus_
 
 static int method_set_locked_hint(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(sd_bus_creds_unrefp) sd_bus_creds *creds = NULL;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uid_t uid;
         int r, b;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "b", &b);
         if (r < 0)
@@ -298,7 +280,7 @@ static int method_set_locked_hint(sd_bus_message *message, void *userdata, sd_bu
                 return r;
 
         if (uid != 0 && uid != s->user->user_record->uid)
-                return sd_bus_error_setf(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may set locked hint");
+                return sd_bus_error_set(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may set locked hint");
 
         session_set_locked_hint(s, b);
 
@@ -306,14 +288,13 @@ static int method_set_locked_hint(sd_bus_message *message, void *userdata, sd_bu
 }
 
 int bus_session_method_kill(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         const char *swho;
         int32_t signo;
         KillWho who;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "si", &swho, &signo);
         if (r < 0)
@@ -353,12 +334,11 @@ int bus_session_method_kill(sd_bus_message *message, void *userdata, sd_bus_erro
 
 static int method_take_control(sd_bus_message *message, void *userdata, sd_bus_error *error) {
         _cleanup_(sd_bus_creds_unrefp) sd_bus_creds *creds = NULL;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         int r, force;
         uid_t uid;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "b", &force);
         if (r < 0)
@@ -373,7 +353,7 @@ static int method_take_control(sd_bus_message *message, void *userdata, sd_bus_e
                 return r;
 
         if (uid != 0 && (force || uid != s->user->user_record->uid))
-                return sd_bus_error_setf(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may take control");
+                return sd_bus_error_set(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may take control");
 
         r = session_set_controller(s, sd_bus_message_get_sender(message), force, true);
         if (r < 0)
@@ -383,13 +363,12 @@ static int method_take_control(sd_bus_message *message, void *userdata, sd_bus_e
 }
 
 static int method_release_control(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
 
         assert(message);
-        assert(s);
 
         if (!session_is_controller(s, sd_bus_message_get_sender(message)))
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
 
         session_drop_controller(s);
 
@@ -397,13 +376,12 @@ static int method_release_control(sd_bus_message *message, void *userdata, sd_bu
 }
 
 static int method_set_type(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         const char *t;
         SessionType type;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "s", &t);
         if (r < 0)
@@ -415,32 +393,55 @@ static int method_set_type(sd_bus_message *message, void *userdata, sd_bus_error
                                          "Invalid session type '%s'", t);
 
         if (!session_is_controller(s, sd_bus_message_get_sender(message)))
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_IN_CONTROL, "You must be in control of this session to set type");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You must be in control of this session to set type");
 
         session_set_type(s, type);
 
         return sd_bus_reply_method_return(message, NULL);
 }
 
+static int method_set_display(sd_bus_message *message, void *userdata, sd_bus_error *error) {
+        Session *s = ASSERT_PTR(userdata);
+        const char *display;
+        int r;
+
+        assert(message);
+
+        r = sd_bus_message_read(message, "s", &display);
+        if (r < 0)
+                return r;
+
+        if (!session_is_controller(s, sd_bus_message_get_sender(message)))
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You must be in control of this session to set display");
+
+        if (!SESSION_TYPE_IS_GRAPHICAL(s->type))
+                return sd_bus_error_set(error, SD_BUS_ERROR_NOT_SUPPORTED, "Setting display is only supported for graphical sessions");
+
+        r = session_set_display(s, display);
+        if (r < 0)
+                return r;
+
+        return sd_bus_reply_method_return(message, NULL);
+}
+
 static int method_take_device(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uint32_t major, minor;
         SessionDevice *sd;
         dev_t dev;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "uu", &major, &minor);
         if (r < 0)
                 return r;
 
         if (!DEVICE_MAJOR_VALID(major) || !DEVICE_MINOR_VALID(minor))
-                return sd_bus_error_setf(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
+                return sd_bus_error_set(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
 
         if (!session_is_controller(s, sd_bus_message_get_sender(message)))
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
 
         dev = makedev(major, minor);
         sd = hashmap_get(s->devices, &dev);
@@ -450,7 +451,7 @@ static int method_take_device(sd_bus_message *message, void *userdata, sd_bus_er
                  * The caller should use dup() if it requires more
                  * than one fd (it would be functionally
                  * equivalent). */
-                return sd_bus_error_setf(error, BUS_ERROR_DEVICE_IS_TAKEN, "Device already taken");
+                return sd_bus_error_set(error, BUS_ERROR_DEVICE_IS_TAKEN, "Device already taken");
 
         r = session_device_new(s, dev, true, &sd);
         if (r < 0)
@@ -473,29 +474,28 @@ error:
 }
 
 static int method_release_device(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uint32_t major, minor;
         SessionDevice *sd;
         dev_t dev;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "uu", &major, &minor);
         if (r < 0)
                 return r;
 
         if (!DEVICE_MAJOR_VALID(major) || !DEVICE_MINOR_VALID(minor))
-                return sd_bus_error_setf(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
+                return sd_bus_error_set(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
 
         if (!session_is_controller(s, sd_bus_message_get_sender(message)))
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
 
         dev = makedev(major, minor);
         sd = hashmap_get(s->devices, &dev);
         if (!sd)
-                return sd_bus_error_setf(error, BUS_ERROR_DEVICE_NOT_TAKEN, "Device not taken");
+                return sd_bus_error_set(error, BUS_ERROR_DEVICE_NOT_TAKEN, "Device not taken");
 
         session_device_free(sd);
         session_save(s);
@@ -504,29 +504,28 @@ static int method_release_device(sd_bus_message *message, void *userdata, sd_bus
 }
 
 static int method_pause_device_complete(sd_bus_message *message, void *userdata, sd_bus_error *error) {
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uint32_t major, minor;
         SessionDevice *sd;
         dev_t dev;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "uu", &major, &minor);
         if (r < 0)
                 return r;
 
         if (!DEVICE_MAJOR_VALID(major) || !DEVICE_MINOR_VALID(minor))
-                return sd_bus_error_setf(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
+                return sd_bus_error_set(error, SD_BUS_ERROR_INVALID_ARGS, "Device major/minor is not valid.");
 
         if (!session_is_controller(s, sd_bus_message_get_sender(message)))
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_IN_CONTROL, "You are not in control of this session");
 
         dev = makedev(major, minor);
         sd = hashmap_get(s->devices, &dev);
         if (!sd)
-                return sd_bus_error_setf(error, BUS_ERROR_DEVICE_NOT_TAKEN, "Device not taken");
+                return sd_bus_error_set(error, BUS_ERROR_DEVICE_NOT_TAKEN, "Device not taken");
 
         session_device_complete_pause(sd);
 
@@ -537,13 +536,12 @@ static int method_set_brightness(sd_bus_message *message, void *userdata, sd_bus
         _cleanup_(sd_bus_creds_unrefp) sd_bus_creds *creds = NULL;
         _cleanup_(sd_device_unrefp) sd_device *d = NULL;
         const char *subsystem, *name, *seat;
-        Session *s = userdata;
+        Session *s = ASSERT_PTR(userdata);
         uint32_t brightness;
         uid_t uid;
         int r;
 
         assert(message);
-        assert(s);
 
         r = sd_bus_message_read(message, "ssu", &subsystem, &name, &brightness);
         if (r < 0)
@@ -556,9 +554,9 @@ static int method_set_brightness(sd_bus_message *message, void *userdata, sd_bus
                 return sd_bus_error_setf(error, SD_BUS_ERROR_INVALID_ARGS, "Not a valid device name %s, refusing.", name);
 
         if (!s->seat)
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_YOUR_DEVICE, "Your session has no seat, refusing.");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_YOUR_DEVICE, "Your session has no seat, refusing.");
         if (s->seat->active != s)
-                return sd_bus_error_setf(error, BUS_ERROR_NOT_YOUR_DEVICE, "Session is not in foreground, refusing.");
+                return sd_bus_error_set(error, BUS_ERROR_NOT_YOUR_DEVICE, "Session is not in foreground, refusing.");
 
         r = sd_bus_query_sender_creds(message, SD_BUS_CREDS_EUID, &creds);
         log_debug_elogind("Sender is%s allowed to change brightness", r < 0 ? " NOT" : "");
@@ -570,7 +568,7 @@ static int method_set_brightness(sd_bus_message *message, void *userdata, sd_bus
                 return r;
 
         if (uid != 0 && uid != s->user->user_record->uid)
-                return sd_bus_error_setf(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may change brightness.");
+                return sd_bus_error_set(error, SD_BUS_ERROR_ACCESS_DENIED, "Only owner of session may change brightness.");
 
         r = sd_device_new_from_subsystem_sysname(&d, subsystem, name);
         log_debug_elogind("Device %s:%s%s opened", subsystem, name, r < 0 ? " could NOT be" : "");
@@ -590,7 +588,7 @@ static int method_set_brightness(sd_bus_message *message, void *userdata, sd_bus
 static int session_object_find(sd_bus *bus, const char *path, const char *interface, void *userdata, void **found, sd_bus_error *error) {
         _cleanup_free_ char *e = NULL;
         sd_bus_message *message;
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         Session *session;
         const char *p;
         int r;
@@ -599,7 +597,6 @@ static int session_object_find(sd_bus *bus, const char *path, const char *interf
         assert(path);
         assert(interface);
         assert(found);
-        assert(m);
 
         p = startswith(path, "/org/freedesktop/login1/session/");
         if (!p)
@@ -640,14 +637,13 @@ static int session_node_enumerator(sd_bus *bus, const char *path, void *userdata
         sd_bus_message *message;
         Manager *m = userdata;
         Session *session;
-        Iterator i;
         int r;
 
         assert(bus);
         assert(path);
         assert(nodes);
 
-        HASHMAP_FOREACH(session, m->sessions, i) {
+        HASHMAP_FOREACH(session, m->sessions) {
                 char *p;
 
                 p = session_bus_path(session);
@@ -758,12 +754,11 @@ int session_send_lock(Session *s, bool lock) {
 
 int session_send_lock_all(Manager *m, bool lock) {
         Session *session;
-        Iterator i;
         int r = 0;
 
         assert(m);
 
-        HASHMAP_FOREACH(session, m->sessions, i) {
+        HASHMAP_FOREACH(session, m->sessions) {
                 int k;
 
                 k = session_send_lock(session, lock);
@@ -892,86 +887,68 @@ static const sd_bus_vtable session_vtable[] = {
                       NULL,
                       bus_session_method_lock,
                       SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetIdleHint",
-                                 "b",
-                                 SD_BUS_PARAM(idle),
-                                 NULL,,
-                                 method_set_idle_hint,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetLockedHint",
-                                 "b",
-                                 SD_BUS_PARAM(locked),
-                                 NULL,,
-                                 method_set_locked_hint,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("Kill",
-                                 "si",
-                                 SD_BUS_PARAM(who)
-                                 SD_BUS_PARAM(signal_number),
-                                 NULL,,
-                                 bus_session_method_kill,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("TakeControl",
-                                 "b",
-                                 SD_BUS_PARAM(force),
-                                 NULL,,
-                                 method_take_control,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetIdleHint",
+                                SD_BUS_ARGS("b", idle),
+                                SD_BUS_NO_RESULT,
+                                method_set_idle_hint,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetLockedHint",
+                                SD_BUS_ARGS("b", locked),
+                                SD_BUS_NO_RESULT,
+                                method_set_locked_hint,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("Kill",
+                                SD_BUS_ARGS("s", who, "i", signal_number),
+                                SD_BUS_NO_RESULT,
+                                bus_session_method_kill,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("TakeControl",
+                                SD_BUS_ARGS("b", force),
+                                SD_BUS_NO_RESULT,
+                                method_take_control,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
         SD_BUS_METHOD("ReleaseControl",
                       NULL,
                       NULL,
                       method_release_control,
                       SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetType",
-                                 "s",
-                                 SD_BUS_PARAM(type),
-                                 NULL,,
-                                 method_set_type,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("TakeDevice",
-                                 "uu",
-                                 SD_BUS_PARAM(major)
-                                 SD_BUS_PARAM(minor),
-                                 "hb",
-                                 SD_BUS_PARAM(fd)
-                                 SD_BUS_PARAM(inactive),
-                                 method_take_device,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("ReleaseDevice",
-                                 "uu",
-                                 SD_BUS_PARAM(major)
-                                 SD_BUS_PARAM(minor),
-                                 NULL,,
-                                 method_release_device,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("PauseDeviceComplete",
-                                 "uu",
-                                 SD_BUS_PARAM(major)
-                                 SD_BUS_PARAM(minor),
-                                 NULL,,
-                                 method_pause_device_complete,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
-        SD_BUS_METHOD_WITH_NAMES("SetBrightness",
-                                 "ssu",
-                                 SD_BUS_PARAM(subsystem)
-                                 SD_BUS_PARAM(name)
-                                 SD_BUS_PARAM(brightness),
-                                 NULL,,
-                                 method_set_brightness,
-                                 SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetType",
+                                SD_BUS_ARGS("s", type),
+                                SD_BUS_NO_RESULT,
+                                method_set_type,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetDisplay",
+                                SD_BUS_ARGS("s", display),
+                                SD_BUS_NO_RESULT,
+                                method_set_display,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("TakeDevice",
+                                SD_BUS_ARGS("u", major, "u", minor),
+                                SD_BUS_RESULT("h", fd, "b", inactive),
+                                method_take_device,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("ReleaseDevice",
+                                SD_BUS_ARGS("u", major, "u", minor),
+                                SD_BUS_NO_RESULT,
+                                method_release_device,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("PauseDeviceComplete",
+                                SD_BUS_ARGS("u", major, "u", minor),
+                                SD_BUS_NO_RESULT,
+                                method_pause_device_complete,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
+        SD_BUS_METHOD_WITH_ARGS("SetBrightness",
+                                SD_BUS_ARGS("s", subsystem, "s", name, "u", brightness),
+                                SD_BUS_NO_RESULT,
+                                method_set_brightness,
+                                SD_BUS_VTABLE_UNPRIVILEGED),
 
-        SD_BUS_SIGNAL_WITH_NAMES("PauseDevice",
-                                 "uus",
-                                 SD_BUS_PARAM(major)
-                                 SD_BUS_PARAM(minor)
-                                 SD_BUS_PARAM(type),
-                                 0),
-        SD_BUS_SIGNAL_WITH_NAMES("ResumeDevice",
-                                 "uuh",
-                                 SD_BUS_PARAM(major)
-                                 SD_BUS_PARAM(minor)
-                                 SD_BUS_PARAM(fd),
-                                 0),
+        SD_BUS_SIGNAL_WITH_ARGS("PauseDevice",
+                                SD_BUS_ARGS("u", major, "u", minor, "s", type),
+                                0),
+        SD_BUS_SIGNAL_WITH_ARGS("ResumeDevice",
+                                SD_BUS_ARGS("u", major, "u", minor, "h", fd),
+                                0),
         SD_BUS_SIGNAL("Lock", NULL, 0),
         SD_BUS_SIGNAL("Unlock", NULL, 0),
 
