@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
 typedef struct Session Session;
@@ -15,7 +15,7 @@ typedef enum SessionState {
         SESSION_ACTIVE,   /* Logged in and in the fg */
         SESSION_CLOSING,  /* Logged out, but scope is still there */
         _SESSION_STATE_MAX,
-        _SESSION_STATE_INVALID = -1
+        _SESSION_STATE_INVALID = -EINVAL,
 } SessionState;
 
 typedef enum SessionClass {
@@ -24,7 +24,7 @@ typedef enum SessionClass {
         SESSION_LOCK_SCREEN,
         SESSION_BACKGROUND,
         _SESSION_CLASS_MAX,
-        _SESSION_CLASS_INVALID = -1
+        _SESSION_CLASS_INVALID = -EINVAL,
 } SessionClass;
 
 typedef enum SessionType {
@@ -35,7 +35,7 @@ typedef enum SessionType {
         SESSION_MIR,
         SESSION_WEB,
         _SESSION_TYPE_MAX,
-        _SESSION_TYPE_INVALID = -1
+        _SESSION_TYPE_INVALID = -EINVAL,
 } SessionType;
 
 #define SESSION_TYPE_IS_GRAPHICAL(type) IN_SET(type, SESSION_X11, SESSION_WAYLAND, SESSION_MIR)
@@ -44,7 +44,7 @@ enum KillWho {
         KILL_LEADER,
         KILL_ALL,
         _KILL_WHO_MAX,
-        _KILL_WHO_INVALID = -1
+        _KILL_WHO_INVALID = -EINVAL,
 };
 
 typedef enum TTYValidity {
@@ -52,7 +52,7 @@ typedef enum TTYValidity {
         TTY_FROM_UTMP,
         TTY_UTMP_INCONSISTENT, /* may happen on ssh sessions with multiplexed TTYs */
         _TTY_VALIDITY_MAX,
-        _TTY_VALIDITY_INVALID = -1,
+        _TTY_VALIDITY_INVALID = -EINVAL,
 } TTYValidity;
 
 struct Session {
@@ -81,7 +81,7 @@ struct Session {
         char *desktop;
 
         char *scope;
-#if 0 /// UNNEEDED by elogind
+#if 0 /// elogind does not support systemd scope jobsd
         char *scope_job;
 #endif // 0
 
@@ -117,6 +117,8 @@ struct Session {
         Hashmap *devices;
         sd_bus_track *track;
 
+        sd_event_source *stop_on_idle_event_source;
+
         LIST_FIELDS(Session, sessions_by_user);
         LIST_FIELDS(Session, sessions_by_seat);
 
@@ -139,6 +141,7 @@ int session_set_idle_hint(Session *s, bool b);
 int session_get_locked_hint(Session *s);
 void session_set_locked_hint(Session *s, bool b);
 void session_set_type(Session *s, SessionType t);
+int session_set_display(Session *s, const char *display);
 int session_create_fifo(Session *s);
 int session_start(Session *s, sd_bus_message *properties, sd_bus_error *error);
 int session_stop(Session *s, bool force);
