@@ -1,39 +1,41 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
 #include <unistd.h>
 
 #include "alloc-util.h"
-#include "bus-common-errors.h"
+//#include "bus-common-errors.h"
 #include "bus-error.h"
-#include "bus-util.h"
-#include "cgroup-util.h"
+//#include "bus-util.h"
+//#include "cgroup-util.h"
 #include "clean-ipc.h"
 #include "env-file.h"
 #include "escape.h"
 #include "fd-util.h"
 #include "fileio.h"
 #include "format-util.h"
-#include "fs-util.h"
+//#include "fs-util.h"
 #include "hashmap.h"
-#include "label.h"
+//#include "label.h"
 #include "limits-util.h"
 #include "logind-dbus.h"
-#include "logind-user.h"
 #include "logind-user-dbus.h"
-#include "mkdir.h"
+#include "logind-user.h"
+#include "mkdir-label.h"
 #include "parse-util.h"
-#include "path-util.h"
+//#include "path-util.h"
+#include "percent-util.h"
 #include "rm-rf.h"
 #include "serialize.h"
 #include "special.h"
 #include "stdio-util.h"
 #include "string-table.h"
-#include "strv.h"
+//#include "strv.h"
 #include "tmpfile-util.h"
+#include "uid-alloc-range.h"
 #include "unit-name.h"
 #include "user-util.h"
-#include "util.h"
+//#include "util.h"
 /// Additional includes needed by elogind
 #include "user-runtime-dir.h"
 
@@ -80,15 +82,16 @@ int user_new(User **ret,
         if (r < 0)
                 return r;
 
+#if 0 /// elogind does not need the systemd runtime_dir_service
         r = unit_name_build("user-runtime-dir", lu, ".service", &u->runtime_dir_service);
         if (r < 0)
                 return r;
+#endif // 0
 
         r = hashmap_put(m->users, UID_TO_PTR(ur->uid), u);
         if (r < 0)
                 return r;
 
-#if 0 /// elogind does not support systemd units
         r = hashmap_put(m->user_units, u->slice, u);
         if (r < 0)
                 return r;
@@ -97,6 +100,7 @@ int user_new(User **ret,
         if (r < 0)
                 return r;
 
+#if 0 /// elogind does not need the systemd runtime_dir_service
         r = hashmap_put(m->user_units, u->runtime_dir_service, u);
         if (r < 0)
                 return r;
@@ -117,27 +121,29 @@ User *user_free(User *u) {
         while (u->sessions)
                 session_free(u->sessions);
 
-#if 0 /// elogind does not support systemd units
         if (u->service)
                 hashmap_remove_value(u->manager->user_units, u->service, u);
 
+#if 0 /// elogind does not need the systemd runtime_dir_service
         if (u->runtime_dir_service)
                 hashmap_remove_value(u->manager->user_units, u->runtime_dir_service, u);
+#endif // 0
 
         if (u->slice)
                 hashmap_remove_value(u->manager->user_units, u->slice, u);
-#endif // 0
 
         hashmap_remove_value(u->manager->users, UID_TO_PTR(u->user_record->uid), u);
 
         sd_event_source_unref(u->timer_event_source);
 
-#if 0 /// elogind neither supports slice nor service jobs.
+#if 0 /// elogind does not support services and service jobs.
         u->service_job = mfree(u->service_job);
 #endif // 0
 
         u->service = mfree(u->service);
+#if 0 /// elogind does not need the systemd runtime_dir_service
         u->runtime_dir_service = mfree(u->runtime_dir_service);
+#endif // 0
         u->slice = mfree(u->slice);
         u->runtime_path = mfree(u->runtime_path);
         u->state_file = mfree(u->state_file);
@@ -178,7 +184,7 @@ static int user_save_internal(User *u) {
         if (u->runtime_path)
                 fprintf(f, "RUNTIME=%s\n", u->runtime_path);
 
-#if 0 /// elogind neither supports service nor slice jobs
+#if 0 /// elogind does not support service jobs.
         if (u->service_job)
                 fprintf(f, "SERVICE_JOB=%s\n", u->service_job);
 
@@ -198,7 +204,6 @@ static int user_save_internal(User *u) {
                         u->last_session_timestamp);
 
         if (u->sessions) {
-                Session *i;
                 bool first;
 
                 fputs("SESSIONS=", f);
@@ -320,7 +325,7 @@ int user_load(User *u) {
         assert(u);
 
         r = parse_env_file(NULL, u->state_file,
-#if 0 /// elogind neither supports service jobs
+#if 0 /// elogind does not support service jobs.
                            "SERVICE_JOB",            &u->service_job,
 #endif // 0
                            "STOPPING",               &stopping,
@@ -374,17 +379,21 @@ static void user_start_service(User *u) {
 }
 #endif // 0
 
+#if 0 /// elogind does not support systemd slices
 static int update_slice_callback(sd_bus_message *m, void *userdata, sd_bus_error *ret_error) {
-        _cleanup_(user_record_unrefp) UserRecord *ur = userdata;
+        _cleanup_(user_record_unrefp) UserRecord *ur = ASSERT_PTR(userdata);
+        const sd_bus_error *e;
+        int r;
 
         assert(m);
-        assert(ur);
 
-        if (sd_bus_message_is_method_error(m, NULL)) {
-                log_warning_errno(sd_bus_message_get_errno(m),
+        e = sd_bus_message_get_error(m);
+        if (e) {
+                r = sd_bus_error_get_errno(e);
+                log_warning_errno(r,
                                   "Failed to update slice of %s, ignoring: %s",
                                   ur->user_name,
-                                  sd_bus_message_get_error(m)->message);
+                                  bus_error_message(e, r));
 
                 return 0;
         }
@@ -409,9 +418,9 @@ static int user_update_slice(User *u) {
         r = sd_bus_message_new_method_call(
                         u->manager->bus,
                         &m,
-                        "org.freedesktop.elogind1",
-                        "/org/freedesktop/elogind1",
-                        "org.freedesktop.elogind1.Manager",
+                        "org.freedesktop.systemd1",
+                        "/org/freedesktop/systemd1",
+                        "org.freedesktop.systemd1.Manager",
                         "SetUnitProperties");
         if (r < 0)
                 return bus_log_create_error(r);
@@ -455,6 +464,7 @@ static int user_update_slice(User *u) {
 
         return 0;
 }
+#endif // 0
 
 int user_start(User *u) {
         assert(u);
@@ -481,8 +491,10 @@ int user_start(User *u) {
          * systemd --user.  We need to do user_save_internal() because we have not "officially" started yet. */
         user_save_internal(u);
 
+#if 0 /// elogind does not support systemd slices
         /* Set slice parameters */
         (void) user_update_slice(u);
+#endif // 0
 
 #if 0 /// elogind does not spawn user instances of systemd
         /* Start user@UID.service */
@@ -503,7 +515,7 @@ int user_start(User *u) {
 }
 
 #if 0 /// elogind does not support user services and systemd units
-static void user_stop_service(User *u) {
+static void user_stop_service(User *u, bool force) {
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
         int r;
 
@@ -515,15 +527,15 @@ static void user_stop_service(User *u) {
 
         u->service_job = mfree(u->service_job);
 
-        r = manager_stop_unit(u->manager, u->service, &error, &u->service_job);
+        r = manager_stop_unit(u->manager, u->service, force ? "replace" : "fail", &error, &u->service_job);
         if (r < 0)
                 log_warning_errno(r, "Failed to stop user service '%s', ignoring: %s", u->service, bus_error_message(&error, r));
 }
 #endif // 0
 
 int user_stop(User *u, bool force) {
-        Session *s;
         int r = 0;
+
         assert(u);
 
         /* This is called whenever we begin with tearing down a user record. It's called in two cases: explicit API
@@ -549,7 +561,7 @@ int user_stop(User *u, bool force) {
         }
 
 #if 0 /// elogind does not support service or slice jobs...
-        user_stop_service(u);
+        user_stop_service(u, force);
 #else // 0
         user_add_to_gc_queue(u);
 #endif // 0
@@ -562,7 +574,6 @@ int user_stop(User *u, bool force) {
 }
 
 int user_finalize(User *u) {
-        Session *s;
         int r = 0, k;
 
         assert(u);
@@ -571,8 +582,11 @@ int user_finalize(User *u) {
          * done. This is called as a result of an earlier user_done() when all jobs are completed. */
 
         if (u->started)
-                log_debug_elogind("User %s not started, finalizing...", u->user_record->user_name);
                 log_debug("User %s logged out.", u->user_record->user_name);
+#if 1 /// let elogind at least issue a debug message if u wasn't even started
+        else
+                log_debug_elogind("User %s not started, finalizing...", u->user_record->user_name);
+#endif // 1
 
         LIST_FOREACH(sessions_by_user, s, u->sessions) {
                 k = session_finalize(s);
@@ -610,7 +624,6 @@ int user_finalize(User *u) {
 }
 
 int user_get_idle_hint(User *u, dual_timestamp *t) {
-        Session *s;
         bool idle_hint = true;
         dual_timestamp ts = DUAL_TIMESTAMP_NULL;
 
@@ -666,7 +679,6 @@ int user_check_linger_file(User *u) {
 
 #if 0 /// UNNEEDED by elogind
 static bool user_unit_active(User *u) {
-        const char *i;
         int r;
 
         assert(u->service);
@@ -678,7 +690,7 @@ static bool user_unit_active(User *u) {
 
                 r = manager_unit_is_active(u->manager, i, &error);
                 if (r < 0)
-                        log_debug_errno(r, "Failed to determine whether unit '%s' is active, ignoring: %s", u->service, bus_error_message(&error, r));
+                        log_debug_errno(r, "Failed to determine whether unit '%s' is active, ignoring: %s", i, bus_error_message(&error, r));
                 if (r != 0)
                         return true;
         }
@@ -700,7 +712,7 @@ static usec_t user_get_stop_delay(User *u) {
 }
 
 bool user_may_gc(User *u, bool drop_not_started) {
-#if 0 /// UNNEEDED by elogind
+#if 0 /// elogind neither supports service nor slice jobs
         int r;
 #endif // 0
 
@@ -774,8 +786,6 @@ void user_add_to_gc_queue(User *u) {
 }
 
 UserState user_get_state(User *u) {
-        Session *i;
-
         assert(u);
 
         if (u->stopping)
@@ -820,13 +830,12 @@ int user_kill(User *u, int signo) {
 
         return manager_kill_unit(u->manager, u->slice, KILL_ALL, signo, NULL);
 #else // 0
-        Session *s;
         int res = 0;
 
         assert(u);
 
-        LIST_FOREACH(sessions_by_user, s, u->sessions) {
-                int r = session_kill(s, KILL_ALL, signo);
+        LIST_FOREACH(sessions_by_user, session, u->sessions) {
+                int r = session_kill(session, KILL_ALL, signo);
                 if (res == 0 && r < 0)
                         res = r;
         }
@@ -881,8 +890,6 @@ static int elect_display_compare(Session *s1, Session *s2) {
 }
 
 void user_elect_display(User *u) {
-        Session *s;
-
         assert(u);
 
         /* This elects a primary session for each user, which we call the "display". We try to keep the assignment
@@ -903,9 +910,8 @@ void user_elect_display(User *u) {
 }
 
 static int user_stop_timeout_callback(sd_event_source *es, uint64_t usec, void *userdata) {
-        User *u = userdata;
+        User *u = ASSERT_PTR(userdata);
 
-        assert(u);
         user_add_to_gc_queue(u);
 
         return 0;
@@ -932,7 +938,7 @@ void user_update_last_session_timer(User *u) {
         assert(!u->timer_event_source);
 
         user_stop_delay = user_get_stop_delay(u);
-        if (IN_SET(user_stop_delay, 0, USEC_INFINITY))
+        if (!timestamp_is_set(user_stop_delay))
                 return;
 
         if (sd_event_get_state(u->manager->event) == SD_EVENT_FINISHED) {
@@ -948,13 +954,10 @@ void user_update_last_session_timer(User *u) {
         if (r < 0)
                 log_warning_errno(r, "Failed to enqueue user stop event source, ignoring: %m");
 
-        if (DEBUG_LOGGING) {
-                char s[FORMAT_TIMESPAN_MAX];
-
+        if (DEBUG_LOGGING)
                 log_debug("Last session of user '%s' logged out, terminating user context in %s.",
                           u->user_record->user_name,
-                          format_timespan(s, sizeof(s), user_stop_delay, USEC_PER_MSEC));
-        }
+                          FORMAT_TIMESPAN(user_stop_delay, USEC_PER_MSEC));
 }
 
 static const char* const user_state_table[_USER_STATE_MAX] = {
@@ -980,18 +983,17 @@ int config_parse_tmpfs_size(
                 void *data,
                 void *userdata) {
 
-        uint64_t *sz = data;
+        uint64_t *sz = ASSERT_PTR(data);
         int r;
 
         assert(filename);
         assert(lvalue);
         assert(rvalue);
-        assert(data);
 
         /* First, try to parse as percentage */
-        r = parse_permille(rvalue);
-        if (r > 0 && r < 1000)
-                *sz = physical_memory_scale(r, 1000U);
+        r = parse_permyriad(rvalue);
+        if (r > 0)
+                *sz = physical_memory_scale(r, 10000U);
         else {
                 uint64_t k;
 
@@ -1001,7 +1003,7 @@ int config_parse_tmpfs_size(
                 if (r >= 0 && (k <= 0 || (uint64_t) (size_t) k != k))
                         r = -ERANGE;
                 if (r < 0) {
-                        log_syntax(unit, LOG_ERR, filename, line, r, "Failed to parse size value '%s', ignoring: %m", rvalue);
+                        log_syntax(unit, LOG_WARNING, filename, line, r, "Failed to parse size value '%s', ignoring: %m", rvalue);
                         return 0;
                 }
 
