@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
 #include <fcntl.h>
@@ -18,6 +18,7 @@
 //#include "def.h"
 #include "device-util.h"
 #include "dirent-util.h"
+#include "escape.h"
 #include "fd-util.h"
 #include "format-util.h"
 #include "fs-util.h"
@@ -27,6 +28,7 @@
 //#include "logind-user-dbus.h"
 #include "logind.h"
 #include "main-func.h"
+#include "mkdir-label.h"
 #include "parse-util.h"
 #include "process-util.h"
 #include "selinux-util.h"
@@ -34,10 +36,10 @@
 #include "signal-util.h"
 #include "strv.h"
 #include "terminal-util.h"
+//#include "udev-util.h"
 /// Additional includes needed by elogind
 #include "label.h"
 #include "musl_missing.h"
-#include "udev-util.h"
 #include "user-util.h"
 
 static Manager* manager_unref(Manager *m);
@@ -57,6 +59,7 @@ static int manager_new(Manager **ret) {
                 .console_active_fd = -1,
 #if 0 /// elogind does not support autospawning of vts
                 .reserve_vt_fd = -1,
+                .enable_wall_messages = true,
                 .idle_action_not_before_usec = now(CLOCK_MONOTONIC),
 #endif // 0
         };
@@ -69,20 +72,21 @@ static int manager_new(Manager **ret) {
         m->inhibitors = hashmap_new(&string_hash_ops);
         m->buttons = hashmap_new(&string_hash_ops);
 
-#if 0 /// elogind does not support units
         m->user_units = hashmap_new(&string_hash_ops);
+#if 0 /// elogind does not support session units
         m->session_units = hashmap_new(&string_hash_ops);
 
         if (!m->devices || !m->seats || !m->sessions || !m->sessions_by_leader || !m->users || !m->inhibitors || !m->buttons || !m->user_units || !m->session_units)
 #else // 0
-        if (!m->devices || !m->seats || !m->sessions || !m->sessions_by_leader || !m->users || !m->inhibitors || !m->buttons)
+        if (!m->devices || !m->seats || !m->sessions || !m->sessions_by_leader || !m->users || !m->inhibitors || !m->buttons || !m->user_units)
 #endif // 0
                 return -ENOMEM;
 
 #if 1 /// elogind needs some more data
         r = elogind_manager_new(m);
-        if (r < 0)
+        if (r < 0) {
                 return r;
+        }
 #endif // 1
         r = sd_event_default(&m->event);
         if (r < 0)
@@ -145,8 +149,8 @@ static Manager* manager_unref(Manager *m) {
         hashmap_free(m->buttons);
         hashmap_free(m->brightness_writers);
 
-#if 0 /// elogind does not support systemd units.
         hashmap_free(m->user_units);
+#if 0 /// elogind does not support systemd session units.
         hashmap_free(m->session_units);
 #endif // 0
 
@@ -159,6 +163,8 @@ static Manager* manager_unref(Manager *m) {
         sd_event_source_unref(m->console_active_event_source);
         sd_event_source_unref(m->lid_switch_ignore_event_source);
 
+        sd_event_source_unref(m->reboot_key_long_press_event_source);
+
 #if ENABLE_UTMP
         sd_event_source_unref(m->utmp_event_source);
 #endif
@@ -188,7 +194,6 @@ static Manager* manager_unref(Manager *m) {
         strv_free(m->kill_only_users);
         strv_free(m->kill_exclude_users);
 
-        free(m->scheduled_shutdown_type);
         free(m->scheduled_shutdown_tty);
         free(m->wall_message);
 #if 0 /// UNNEEDED by elogind
@@ -267,7 +272,6 @@ static int manager_enumerate_buttons(Manager *m) {
 
 static int manager_enumerate_seats(Manager *m) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r = 0;
 
         assert(m);
@@ -295,8 +299,8 @@ static int manager_enumerate_seats(Manager *m) {
                 if (!s) {
                         log_debug_elogind("Removing stale seat at /run/systemd/seats/%s", de->d_name);
                         if (unlinkat(dirfd(d), de->d_name, 0) < 0)
-                                log_warning("Failed to remove /run/systemd/seats/%s: %m",
-                                            de->d_name);
+                                log_warning_errno(errno, "Failed to remove /run/systemd/seats/%s, ignoring: %m",
+                                                  de->d_name);
                         continue;
                 }
 
@@ -311,7 +315,6 @@ static int manager_enumerate_seats(Manager *m) {
 
 static int manager_enumerate_linger_users(Manager *m) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r = 0;
 
         assert(m);
@@ -326,12 +329,16 @@ static int manager_enumerate_linger_users(Manager *m) {
 
         FOREACH_DIRENT(de, d, return -errno) {
                 int k;
+                _cleanup_free_ char *n = NULL;
 
-                dirent_ensure_type(d, de);
                 if (!dirent_is_file(de))
                         continue;
-
-                k = manager_add_user_by_name(m, de->d_name, NULL);
+                k = cunescape(de->d_name, 0, &n);
+                if (k < 0) {
+                        r = log_warning_errno(k, "Failed to unescape username '%s', ignoring: %m", de->d_name);
+                        continue;
+                }
+                k = manager_add_user_by_name(m, n, NULL);
                 if (k < 0)
                         r = log_warning_errno(k, "Couldn't add lingering user %s, ignoring: %m", de->d_name);
         }
@@ -341,7 +348,6 @@ static int manager_enumerate_linger_users(Manager *m) {
 
 static int manager_enumerate_users(Manager *m) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r, k;
 
         assert(m);
@@ -377,7 +383,7 @@ static int manager_enumerate_users(Manager *m) {
                         continue;
                 }
 
-                log_debug_elogind("Loading user %d \"%s\"", u->user_record->uid, strna(u->user_record->user_name) );
+                log_debug_elogind("Loading user %u \"%s\"", u->user_record->uid, strna(u->user_record->user_name));
                 user_add_to_gc_queue(u);
 
                 k = user_load(u);
@@ -485,13 +491,9 @@ static int manager_attach_fds(Manager *m) {
                 if (deliver_fd(m, fdnames[i], fd) >= 0)
                         continue;
 
-                /* Hmm, we couldn't deliver the fd to any session device object? If so, let's close the fd */
-                safe_close(fd);
-
-                /* Remove from fdstore as well */
-                (void) sd_notifyf(false,
-                                  "FDSTOREREMOVE=1\n"
-                                  "FDNAME=%s", fdnames[i]);
+                /* Hmm, we couldn't deliver the fd to any session device object? If so, let's close the fd
+                 * and remove it from fdstore. */
+                close_and_notify_warn(fd, fdnames[i]);
         }
 
         return 0;
@@ -499,7 +501,6 @@ static int manager_attach_fds(Manager *m) {
 
 static int manager_enumerate_sessions(Manager *m) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r = 0, k;
 
         assert(m);
@@ -543,7 +544,6 @@ static int manager_enumerate_sessions(Manager *m) {
 
 static int manager_enumerate_inhibitors(Manager *m) {
         _cleanup_closedir_ DIR *d = NULL;
-        struct dirent *de;
         int r = 0;
 
         assert(m);
@@ -579,9 +579,8 @@ static int manager_enumerate_inhibitors(Manager *m) {
 }
 
 static int manager_dispatch_seat_udev(sd_device_monitor *monitor, sd_device *device, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
 
-        assert(m);
         assert(device);
 
         manager_process_seat_device(m, device);
@@ -589,9 +588,8 @@ static int manager_dispatch_seat_udev(sd_device_monitor *monitor, sd_device *dev
 }
 
 static int manager_dispatch_device_udev(sd_device_monitor *monitor, sd_device *device, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
 
-        assert(m);
         assert(device);
 
         manager_process_seat_device(m, device);
@@ -600,10 +598,9 @@ static int manager_dispatch_device_udev(sd_device_monitor *monitor, sd_device *d
 
 #if 0 /// UNNEEDED by elogind
 static int manager_dispatch_vcsa_udev(sd_device_monitor *monitor, sd_device *device, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         const char *name;
 
-        assert(m);
         assert(device);
 
         /* Whenever a VCSA device is removed try to reallocate our
@@ -611,7 +608,7 @@ static int manager_dispatch_vcsa_udev(sd_device_monitor *monitor, sd_device *dev
 
         if (sd_device_get_sysname(device, &name) >= 0 &&
             startswith(name, "vcsa") &&
-            device_for_action(device, DEVICE_ACTION_REMOVE))
+            device_for_action(device, SD_DEVICE_REMOVE))
                 seat_preallocate_vts(m->seat0);
 
         return 0;
@@ -619,9 +616,8 @@ static int manager_dispatch_vcsa_udev(sd_device_monitor *monitor, sd_device *dev
 #endif // 0
 
 static int manager_dispatch_button_udev(sd_device_monitor *monitor, sd_device *device, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
 
-        assert(m);
         assert(device);
 
         manager_process_button_device(m, device);
@@ -629,9 +625,8 @@ static int manager_dispatch_button_udev(sd_device_monitor *monitor, sd_device *d
 }
 
 static int manager_dispatch_console(sd_event_source *s, int fd, uint32_t revents, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
 
-        assert(m);
         assert(m->seat0);
         assert(m->console_active_fd == fd);
 
@@ -709,6 +704,13 @@ static int manager_connect_bus(Manager *m) {
         r = bus_call_method_async(m->bus, NULL, bus_systemd_mgr, "Subscribe", NULL, NULL, NULL);
         if (r < 0)
                 return log_error_errno(r, "Failed to enable subscription: %m");
+#else // 0
+        /* Since the facilities just above are not available, elogind needs another mean to
+         * take action when the dbus connection is closed.
+         */
+        r = sd_bus_set_exit_on_disconnect(m->bus, true);
+        if (r < 0)
+                return log_error_errno(r, "Failed to set exit on disconnect: %m");
 #endif // 0
 
         r = sd_bus_request_name_async(m->bus, NULL, "org.freedesktop.login1", 0, NULL, NULL);
@@ -730,7 +732,7 @@ static int manager_connect_bus(Manager *m) {
 
 static int manager_vt_switch(sd_event_source *src, const struct signalfd_siginfo *si, void *data) {
         Manager *m = data;
-        Session *active, *iter;
+        Session *active;
 
         /*
          * We got a VT-switch signal and we have to acknowledge it immediately.
@@ -777,16 +779,14 @@ static int manager_vt_switch(sd_event_source *src, const struct signalfd_siginfo
                 return 0;
         }
 
-        if (active->vtfd >= 0) {
+        if (active->vtfd >= 0)
                 session_leave_vt(active);
-        } else {
-                LIST_FOREACH(sessions_by_seat, iter, m->seat0->sessions) {
+        else
+                LIST_FOREACH(sessions_by_seat, iter, m->seat0->sessions)
                         if (iter->vtnr == active->vtnr && iter->vtfd >= 0) {
                                 session_leave_vt(iter);
                                 break;
                         }
-                }
-        }
 
         return 0;
 }
@@ -829,10 +829,10 @@ static int manager_connect_console(Manager *m) {
 
         if (SIGRTMIN + 1 > SIGRTMAX)
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
-                                       "Not enough real-time signals available: %u-%u",
+                                       "Not enough real-time signals available: %i-%i",
                                        SIGRTMIN, SIGRTMAX);
 
-        assert_se(ignore_signals(SIGRTMIN + 1, -1) >= 0);
+        assert_se(ignore_signals(SIGRTMIN + 1) >= 0);
         assert_se(sigprocmask_many(SIG_BLOCK, NULL, SIGRTMIN, -1) >= 0);
 
         r = sd_event_add_signal(m->event, NULL, SIGRTMIN, manager_vt_switch, m);
@@ -867,7 +867,7 @@ static int manager_connect_udev(Manager *m) {
         if (r < 0)
                 return r;
 
-        (void) sd_event_source_set_description(sd_device_monitor_get_event_source(m->device_seat_monitor), "logind-seat-monitor");
+        (void) sd_device_monitor_set_description(m->device_seat_monitor, "seat");
 
         r = sd_device_monitor_new(&m->device_monitor);
         if (r < 0)
@@ -893,7 +893,7 @@ static int manager_connect_udev(Manager *m) {
         if (r < 0)
                 return r;
 
-        (void) sd_event_source_set_description(sd_device_monitor_get_event_source(m->device_monitor), "logind-device-monitor");
+        (void) sd_device_monitor_set_description(m->device_monitor, "input,graphics,drm");
 
         /* Don't watch keys if nobody cares */
         if (!manager_all_buttons_ignored(m)) {
@@ -917,7 +917,7 @@ static int manager_connect_udev(Manager *m) {
                 if (r < 0)
                         return r;
 
-                (void) sd_event_source_set_description(sd_device_monitor_get_event_source(m->device_button_monitor), "logind-button-monitor");
+                (void) sd_device_monitor_set_description(m->device_button_monitor, "button");
         }
 
 #if 0 /// elogind does not support autospawning of vts
@@ -940,7 +940,7 @@ static int manager_connect_udev(Manager *m) {
                 if (r < 0)
                         return r;
 
-                (void) sd_event_source_set_description(sd_device_monitor_get_event_source(m->device_vcsa_monitor), "logind-vcsa-monitor");
+                (void) sd_device_monitor_set_description(m->device_vcsa_monitor, "vcsa");
         }
 #endif // 0
 
@@ -959,7 +959,7 @@ static void manager_gc(Manager *m, bool drop_not_started) {
                 seat->in_gc_queue = false;
 
                 if (seat_may_gc(seat, drop_not_started)) {
-                        seat_stop(seat, false);
+                        seat_stop(seat, /* force = */ false);
                         seat_free(seat);
                 }
         }
@@ -968,14 +968,13 @@ static void manager_gc(Manager *m, bool drop_not_started) {
                 LIST_REMOVE(gc_queue, m->session_gc_queue, session);
                 session->in_gc_queue = false;
 
-                /* First, if we are not closing yet, initiate stopping */
+                /* First, if we are not closing yet, initiate stopping. */
                 if (session_may_gc(session, drop_not_started) &&
                     session_get_state(session) != SESSION_CLOSING)
-                        (void) session_stop(session, false);
+                        (void) session_stop(session, /* force = */ false);
 
-                /* Normally, this should make the session referenced
-                 * again, if it doesn't then let's get rid of it
-                 * immediately */
+                /* Normally, this should make the session referenced again, if it doesn't then let's get rid
+                 * of it immediately. */
                 if (session_may_gc(session, drop_not_started)) {
                         (void) session_finalize(session);
                         session_free(session);
@@ -999,13 +998,11 @@ static void manager_gc(Manager *m, bool drop_not_started) {
 }
 
 static int manager_dispatch_idle_action(sd_event_source *s, uint64_t t, void *userdata) {
-        Manager *m = userdata;
+        Manager *m = ASSERT_PTR(userdata);
         struct dual_timestamp since;
         usec_t n, elapse;
         int r;
 
-        assert(m);
-
         if (m->idle_action == HANDLE_IGNORE ||
             m->idle_action_usec <= 0)
                 return 0;
@@ -1013,18 +1010,33 @@ static int manager_dispatch_idle_action(sd_event_source *s, uint64_t t, void *us
         n = now(CLOCK_MONOTONIC);
 
         r = manager_get_idle_hint(m, &since);
-        if (r <= 0)
+        if (r <= 0) {
                 /* Not idle. Let's check if after a timeout it might be idle then. */
                 elapse = n + m->idle_action_usec;
-        else {
+                m->was_idle = false;
+        } else {
+
                 /* Idle! Let's see if it's time to do something, or if
                  * we shall sleep for longer. */
 
                 if (n >= since.monotonic + m->idle_action_usec &&
                     (m->idle_action_not_before_usec <= 0 || n >= m->idle_action_not_before_usec + m->idle_action_usec)) {
-                        log_info("System idle. Doing %s operation.", handle_action_to_string(m->idle_action));
+                        bool is_edge = false;
 
-                        manager_handle_action(m, 0, m->idle_action, false, false);
+                        /* We weren't idle previously or some activity happened while we were sleeping, and now we are
+                         * idle. Let's remember that for the next time and make this an edge transition. */
+                        if (!m->was_idle || since.monotonic >= m->idle_action_not_before_usec) {
+                                is_edge = true;
+                                m->was_idle = true;
+                        }
+
+                        if (m->idle_action == HANDLE_LOCK && !is_edge)
+                                /* We are idle and we were before so we are actually not taking any action. */
+                                log_debug("System idle.");
+                        else
+                                log_info("System idle. Will %s now.", handle_action_verb_to_string(m->idle_action));
+
+                        manager_handle_action(m, 0, m->idle_action, false, is_edge);
                         m->idle_action_not_before_usec = n;
                 }
 
@@ -1069,9 +1081,6 @@ static int manager_dispatch_reload_signal(sd_event_source *s, const struct signa
         else
                 log_info("Config file reloaded.");
 
-#if 1 /// elogind needs an Add-On for sleep configuration
-        elogind_manager_reset_config(m);
-#endif // 1
         return 0;
 }
 
@@ -1082,7 +1091,6 @@ static int manager_startup(Manager *m) {
         User *user;
         Button *button;
         Inhibitor *inhibitor;
-        Iterator i;
 
         assert(m);
 
@@ -1092,8 +1100,9 @@ static int manager_startup(Manager *m) {
 
 #if 1 /// install elogind specific signal handlers
         r = elogind_manager_startup(m);
-        if (r < 0)
+        if (r < 0) {
                 return log_error_errno(r, "Failed to register elogind signal handlers: %m");
+        }
 #endif // 1
         /* Connect to utmp */
         manager_connect_utmp(m);
@@ -1147,6 +1156,8 @@ static int manager_startup(Manager *m) {
         if (r < 0)
                 log_warning_errno(r, "Button enumeration failed: %m");
 
+        manager_load_scheduled_shutdown(m);
+
         /* Remove stale objects before we start them */
         manager_gc(m, false);
 
@@ -1159,16 +1170,16 @@ static int manager_startup(Manager *m) {
         manager_read_utmp(m);
 
         /* And start everything */
-        HASHMAP_FOREACH(seat, m->seats, i)
+        HASHMAP_FOREACH(seat, m->seats)
                 (void) seat_start(seat);
 
-        HASHMAP_FOREACH(user, m->users, i)
+        HASHMAP_FOREACH(user, m->users)
                 (void) user_start(user);
 
-        HASHMAP_FOREACH(session, m->sessions, i)
+        HASHMAP_FOREACH(session, m->sessions)
                 (void) session_start(session, NULL, NULL);
 
-        HASHMAP_FOREACH(inhibitor, m->inhibitors, i) {
+        HASHMAP_FOREACH(inhibitor, m->inhibitors) {
                 (void) inhibitor_start(inhibitor);
 
                 /* Let's see if the inhibitor is dead now, then remove it */
@@ -1178,7 +1189,7 @@ static int manager_startup(Manager *m) {
                 }
         }
 
-        HASHMAP_FOREACH(button, m->buttons, i)
+        HASHMAP_FOREACH(button, m->buttons)
                 button_check_switches(button);
 
         manager_dispatch_idle_action(NULL, 0, m);
@@ -1192,6 +1203,11 @@ static int manager_run(Manager *m) {
         assert(m);
 
         for (;;) {
+#if 1 /// Only issue a debug message if elogind is restarted through an INTerrupt
+                if (m->do_interrupt) {
+                        log_debug_elogind("elogind interrupted (PID " PID_FMT "), going down silently...", getpid_cached());
+                }
+#endif // 1
                 r = sd_event_get_state(m->event);
                 if (r < 0)
                         return r;
@@ -1206,7 +1222,7 @@ static int manager_run(Manager *m) {
                 if (r > 0)
                         continue;
 
-                r = sd_event_run(m->event, (uint64_t) -1);
+                r = sd_event_run(m->event, UINT64_MAX);
                 if (r < 0)
                         return r;
         }
@@ -1214,19 +1230,20 @@ static int manager_run(Manager *m) {
 
 static int run(int argc, char *argv[]) {
         _cleanup_(manager_unrefp) Manager *m = NULL;
-        _cleanup_(notify_on_cleanup) const char *notify_message = NULL;
+        _unused_ _cleanup_(notify_on_cleanup) const char *notify_message = NULL;
         int r;
 
         elogind_set_program_name(argv[0]);
         log_set_facility(LOG_AUTH);
-        log_setup_service();
+        log_setup();
 
 #if 1 /// perform extra checks for elogind startup, and fork if wanted
         bool has_forked = false;
         r = elogind_startup(argc, argv, &has_forked);
         log_debug_elogind("elogind_startup() exited with %d", r);
-        if (r)
+        if (r) {
                 return r < 0 ? EXIT_FAILURE : EXIT_SUCCESS;
+        }
         // If we forked, we are in the grandchild, the daemon, now.
 #endif // 1
 
@@ -1245,8 +1262,9 @@ static int run(int argc, char *argv[]) {
                                        BUS_IMPLEMENTATIONS(&manager_object, &log_control_object),
                                        argc, argv);
                 log_debug_elogind("service_parse_argv() returned %d", r);
-                if (r <= 0)
+                if (r <= 0) {
                         return r;
+                }
         }
 #endif // 0
 
@@ -1256,29 +1274,34 @@ static int run(int argc, char *argv[]) {
         if (r < 0)
                 return r;
 
-        /* Always create the directories people can create inotify watches in. Note that some applications might check
-         * for the existence of /run/systemd/seats/ to determine whether logind is available, so please always make
-         * sure these directories are created early on and unconditionally. */
 #if 0 /// elogind can not rely on systemd to help, so we need a bit more effort than this
+        /* Always create the directories people can create inotify watches in. Note that some applications
+         * might check for the existence of /run/systemd/seats/ to determine whether logind is available, so
+         * please always make sure these directories are created early on and unconditionally. */
         (void) mkdir_label("/run/systemd/seats", 0755);
         (void) mkdir_label("/run/systemd/users", 0755);
         (void) mkdir_label("/run/systemd/sessions", 0755);
 #else // 0
         r = mkdir_label("/run/systemd", 0755);
-        if ( (r < 0) && (-EEXIST != r) )
+        if ((r < 0) && (-EEXIST != r)) {
                 return log_error_errno(r, "Failed to create /run/systemd : %m");
+        }
         r = mkdir_label("/run/systemd/seats", 0755);
-        if ( r < 0 && (-EEXIST != r) )
+        if (r < 0 && (-EEXIST != r)) {
                 return log_error_errno(r, "Failed to create /run/systemd/seats : %m");
+        }
         r = mkdir_label("/run/systemd/users", 0755);
-        if ( r < 0 && (-EEXIST != r) )
+        if (r < 0 && (-EEXIST != r)) {
                 return log_error_errno(r, "Failed to create /run/systemd/users : %m");
+        }
         r = mkdir_label("/run/systemd/sessions", 0755);
-        if ( r < 0 && (-EEXIST != r) )
+        if (r < 0 && (-EEXIST != r)) {
                 return log_error_errno(r, "Failed to create /run/systemd/sessions : %m");
+        }
         r = mkdir_label("/run/systemd/machines", 0755);
-        if ( r < 0 && (-EEXIST != r) )
+        if (r < 0 && (-EEXIST != r)) {
                 return log_error_errno(r, "Failed to create /run/systemd/machines : %m");
+        }
 #endif // 0
 
 #if 0 /// elogind also blocks SIGQUIT, and installs a signal handler for it
@@ -1294,25 +1317,12 @@ static int run(int argc, char *argv[]) {
 
         (void) manager_parse_config_file(m);
 
-#if 1 /// elogind needs an Add-On for sleep configuration
-        elogind_manager_reset_config(m);
-#endif // 1
         log_debug_elogind("%s", "Starting manager...");
         r = manager_startup(m);
         if (r < 0)
                 return log_error_errno(r, "Failed to fully start up daemon: %m");
 
-
-#if 1 /// Do not tell anybody if elogind is restarted through an INTerrupt
-        if (m->do_interrupt) {
-                log_debug_elogind("elogind interrupted (PID "PID_FMT"), going down silently...", getpid_cached());
-        } else {
-#endif /// 1
         notify_message = notify_start(NOTIFY_READY, NOTIFY_STOPPING);
-#if 1 /// extra bracket for elogind here
-        }
-#endif // 1
-
         return manager_run(m);
 }
 
