@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 #pragma once
 
 #include <stdbool.h>
@@ -26,6 +26,7 @@ typedef struct Manager Manager;
 #include "cgroup-util.h"
 #include "elogind.h"
 #include "musl_missing.h"
+#include "sleep-config.h"
 
 #if 1 /// elogind has to ident itself
 #define MANAGER_IS_SYSTEM(m)   (  (m)->is_system)
@@ -91,42 +92,29 @@ struct Manager {
         unsigned long session_counter;
         unsigned long inhibit_counter;
 
-#if 0 /// elogind does not support units
+#if 0 /// elogind does not support session units
         Hashmap *session_units;
-        Hashmap *user_units;
 #endif // 0
+        Hashmap *user_units;
 
         usec_t inhibit_delay_max;
         usec_t user_stop_delay;
 
-        /* If an action is currently being executed or is delayed,
-         * this is != 0 and encodes what is being done */
-        InhibitWhat action_what;
+        /* If a shutdown/suspend was delayed due to an inhibitor this contains the action we are supposed to
+         * start after the delay is over */
+        const HandleActionData *delayed_action;
 
 #if 0 /// elogind does all relevant actions on its own. No systemd jobs and units.
-        /* If a shutdown/suspend was delayed due to a inhibitor this
-           contains the unit name we are supposed to start after the
-           delay is over */
-        const char *action_unit;
-
-        /* If a shutdown/suspend is currently executed, then this is
-         * the job of it */
+        /* If a shutdown/suspend is currently executed, then this is the job of it */
         char *action_job;
 #else // 0
         /* Suspension and hibernation can be disabled in logind.conf. */
-        bool allow_suspend;         /* AllowSuspend */
-        bool allow_hibernate;       /* AllowHibernation */
-        bool allow_s2h;             /* AllowSuspendThenHibernate */
-        bool allow_hybrid_sleep;    /* AllowHybridSleep */
+        bool allow[_SLEEP_OPERATION_MAX];
+        char **modes[_SLEEP_OPERATION_MAX];
+        char **states[_SLEEP_OPERATION_MAX];
+        usec_t hibernate_delay_usec; /* HibernateDelaySec */
+        usec_t suspend_estimation_usec; /* SuspendEstimationSec */
 
-        char **suspend_modes;       /* SuspendMode */
-        char **suspend_states;      /* SuspendState */
-        char **hibernate_modes;     /* HibernateMode */
-        char **hibernate_states;    /* HibernateState */
-        char **hybrid_modes;        /* HybridSleepMode */
-        char **hybrid_states;       /* HybridSleepState */
-
-        usec_t hibernate_delay_sec; /* HibernateDelaySec */
 
         /* If an admin puts scripts into SYSTEM_SLEEP_PATH and/or
            SYSTEM_POWEROFF_PATH that fail, the ongoing suspend/poweroff
@@ -138,18 +126,17 @@ struct Manager {
         /* Allow elogind to put Nvidia cards to sleep */
         bool handle_nvidia_sleep;
 
-        /* If a shutdown/suspend was delayed due to a inhibitor this
-           contains the action we are supposed to perform after the
-           delay is over */
-        HandleAction pending_action;
-
         /* To allow elogind to put nvidia cards to sleep on suspend/hibernate,
            we store the users uid to get the right VT information */
         uid_t scheduled_sleep_uid;
+
+        /* Allow users to set programs which do the suspend/hibernation */
+        char **suspend_by_using;
+        char **hibernate_by_using;
 #endif // 0
         sd_event_source *inhibit_timeout_source;
 
-        char *scheduled_shutdown_type;
+        const HandleActionData *scheduled_shutdown_action;
         usec_t scheduled_shutdown_timeout;
         sd_event_source *scheduled_shutdown_timeout_source;
         uid_t scheduled_shutdown_uid;
@@ -158,7 +145,7 @@ struct Manager {
         bool unlink_nologin;
 
         char *wall_message;
-        unsigned enable_wall_messages;
+        bool enable_wall_messages;
         sd_event_source *wall_message_timeout_source;
 
         bool shutdown_dry_run;
@@ -167,10 +154,19 @@ struct Manager {
         usec_t idle_action_usec;
         usec_t idle_action_not_before_usec;
         HandleAction idle_action;
+        bool was_idle;
+
+        usec_t stop_idle_session_usec;
 
         HandleAction handle_power_key;
+        HandleAction handle_power_key_long_press;
+        HandleAction handle_reboot_key;
+        HandleAction handle_reboot_key_long_press;
         HandleAction handle_suspend_key;
+        HandleAction handle_suspend_key_long_press;
         HandleAction handle_hibernate_key;
+        HandleAction handle_hibernate_key_long_press;
+
         HandleAction handle_lid_switch;
         HandleAction handle_lid_switch_ep;
         HandleAction handle_lid_switch_docked;
@@ -179,6 +175,7 @@ struct Manager {
         bool suspend_key_ignore_inhibited;
         bool hibernate_key_ignore_inhibited;
         bool lid_switch_ignore_inhibited;
+        bool reboot_key_ignore_inhibited;
 
         bool remove_ipc;
 
@@ -187,6 +184,11 @@ struct Manager {
         usec_t holdoff_timeout_usec;
         sd_event_source *lid_switch_ignore_event_source;
 
+        sd_event_source *power_key_long_press_event_source;
+        sd_event_source *reboot_key_long_press_event_source;
+        sd_event_source *suspend_key_long_press_event_source;
+        sd_event_source *hibernate_key_long_press_event_source;
+
         uint64_t runtime_dir_size;
         uint64_t runtime_dir_inodes;
         uint64_t sessions_max;
@@ -214,7 +216,9 @@ int manager_add_inhibitor(Manager *m, const char* id, Inhibitor **ret_inhibitor)
 int manager_process_seat_device(Manager *m, sd_device *d);
 int manager_process_button_device(Manager *m, sd_device *d);
 
+#if 0 /// elogind does not spawn VTs.
 int manager_spawn_autovt(Manager *m, unsigned vtnr);
+#endif // 0
 
 bool manager_shall_kill(Manager *m, const char *user);
 
@@ -241,6 +245,6 @@ CONFIG_PARSER_PROTOTYPE(config_parse_n_autovts);
 CONFIG_PARSER_PROTOTYPE(config_parse_tmpfs_size);
 
 int manager_setup_wall_message_timer(Manager *m);
-bool logind_wall_tty_filter(const char *tty, void *userdata);
+bool logind_wall_tty_filter(const char *tty, bool is_local, void *userdata);
 
 int manager_read_efi_boot_loader_entries(Manager *m);
