@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <fcntl.h>
 
@@ -14,7 +14,6 @@
 #include "strv.h"
 #include "tmpfile-util.h"
 
-#if 0 /// UNNEEDED by elogind
 int serialize_item(FILE *f, const char *key, const char *value) {
         assert(f);
         assert(key);
@@ -79,6 +78,7 @@ int serialize_item_format(FILE *f, const char *key, const char *format, ...) {
         return 1;
 }
 
+#if 0 /// UNNEEDED by elogind
 int serialize_fd(FILE *f, FDSet *fds, const char *key, int fd) {
         int copy;
 
@@ -94,6 +94,7 @@ int serialize_fd(FILE *f, FDSet *fds, const char *key, int fd) {
 
         return serialize_item_format(f, key, "%i", copy);
 }
+#endif // 0
 
 int serialize_usec(FILE *f, const char *key, usec_t usec) {
         assert(f);
@@ -105,6 +106,7 @@ int serialize_usec(FILE *f, const char *key, usec_t usec) {
         return serialize_item_format(f, key, USEC_FMT, usec);
 }
 
+#if 0 /// UNNEEDED by elogind
 int serialize_dual_timestamp(FILE *f, const char *name, const dual_timestamp *t) {
         assert(f);
         assert(name);
@@ -118,7 +120,6 @@ int serialize_dual_timestamp(FILE *f, const char *name, const dual_timestamp *t)
 
 int serialize_strv(FILE *f, const char *key, char **l) {
         int ret = 0, r;
-        char **i;
 
         /* Returns the first error, or positive if anything was serialized, 0 otherwise. */
 
@@ -179,6 +180,7 @@ int deserialize_dual_timestamp(const char *value, dual_timestamp *t) {
 
 int deserialize_environment(const char *value, char ***list) {
         _cleanup_free_ char *unescaped = NULL;
+        ssize_t l;
         int r;
 
         assert(value);
@@ -186,15 +188,14 @@ int deserialize_environment(const char *value, char ***list) {
 
         /* Changes the *environment strv inline. */
 
-        r = cunescape(value, 0, &unescaped);
-        if (r < 0)
-                return log_error_errno(r, "Failed to unescape: %m");
+        l = cunescape(value, 0, &unescaped);
+        if (l < 0)
+                return log_error_errno(l, "Failed to unescape: %m");
 
-        r = strv_env_replace(list, unescaped);
+        r = strv_env_replace_consume(list, TAKE_PTR(unescaped));
         if (r < 0)
                 return log_error_errno(r, "Failed to append environment variable: %m");
 
-        unescaped = NULL; /* now part of 'list' */
         return 0;
 }
 #endif // 0
