@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <unistd.h>
 
@@ -10,11 +10,10 @@
 #include "string-util.h"
 #include "tests.h"
 
-static void test_cg_split_spec(void) {
+#if 0 /// UNNEEDED by elogind
+TEST(cg_split_spec) {
         char *c, *p;
 
-        log_info("/* %s */", __func__);
-
         assert_se(cg_split_spec("foobar:/", &c, &p) == 0);
         assert_se(streq(c, "foobar"));
         assert_se(streq(p, "/"));
@@ -41,16 +40,17 @@ static void test_cg_split_spec(void) {
         assert_se(p == NULL);
         c = mfree(c);
 }
+#endif // 0
 
-static void test_cg_create(void) {
-        log_info("/* %s */", __func__);
+TEST(cg_create) {
         int r;
 
         r = cg_unified_cached(false);
-        if (r < 0) {
-                log_info_errno(r, "Skipping %s: %m", __func__);
+        if (r == -ENOMEDIUM) {
+                log_tests_skipped("cgroup not mounted");
                 return;
         }
+        assert_se(r >= 0);
 
         _cleanup_free_ char *here = NULL;
         assert_se(cg_pid_get_path_shifted(0, NULL, &here) >= 0);
@@ -139,11 +139,4 @@ static void test_cg_create(void) {
         assert_se(cg_rmdir(SYSTEMD_CGROUP_CONTROLLER, test_a) == 0);
 }
 
-int main(int argc, char *argv[]) {
-        test_setup_logging(LOG_DEBUG);
-
-        test_cg_split_spec();
-        test_cg_create();
-
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_DEBUG);
