@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <stdio.h>
 //#include <sys/stat.h>
@@ -8,14 +8,14 @@
 #include "alloc-util.h"
 #include "macro.h"
 #include "pretty-print.h"
-#include "strv.h"
+//#include "strv.h"
 #include "tests.h"
 
-static void test_terminal_urlify(void) {
+TEST(terminal_urlify) {
         _cleanup_free_ char *formatted = NULL;
 
 #if 0 /// Aww... lets use an elogind URL, okay?
-        assert_se(terminal_urlify("https://www.freedesktop.org/wiki/Software/systemd/", "systemd homepage", &formatted) >= 0);
+        assert_se(terminal_urlify("https://www.freedesktop.org/wiki/Software/systemd", "systemd homepage", &formatted) >= 0);
 #else // 0
         assert_se(terminal_urlify("https://github.com/elogind/elogind/", "elogind homepage", &formatted) >= 0);
 #endif // 0
@@ -30,7 +30,7 @@ static void test_terminal_urlify(void) {
 }
 
 #if 0 /// UNNEEDED by elogind
-static void test_cat_files(void) {
+TEST(cat_files) {
         assert_se(cat_files("/no/such/file", NULL, 0) == -ENOENT);
         assert_se(cat_files("/no/such/file", NULL, CAT_FLAGS_MAIN_FILE_OPTIONAL) == 0);
 
@@ -39,15 +39,23 @@ static void test_cat_files(void) {
 }
 #endif // 0
 
-int main(int argc, char *argv[]) {
-        test_setup_logging(LOG_INFO);
+TEST(red_green_cross_check_mark) {
+        bool b = false;
+
+        printf("yea: <%s>\n", GREEN_CHECK_MARK());
+        printf("nay: <%s>\n", RED_CROSS_MARK());
+
+        printf("%s → %s → %s → %s\n",
+               COLOR_MARK_BOOL(b),
+               COLOR_MARK_BOOL(!b),
+               COLOR_MARK_BOOL(!!b),
+               COLOR_MARK_BOOL(!!!b));
+}
 
-        test_terminal_urlify();
 #if 0 /// UNNEEDED by elogind
-        test_cat_files();
-
+TEST(print_separator) {
         print_separator();
+}
 #endif // 0
 
-        return 0;
-}
+DEFINE_TEST_MAIN(LOG_INFO);
