@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include "alloc-util.h"
 #include "env-util.h"
@@ -21,16 +21,12 @@ static int parse_item(const char *key, const char *value, void *data) {
         return 0;
 }
 
-static void test_proc_cmdline_parse(void) {
-        log_info("/* %s */", __func__);
-
+TEST(proc_cmdline_parse) {
         assert_se(proc_cmdline_parse(parse_item, &obj, PROC_CMDLINE_STRIP_RD_PREFIX) >= 0);
 }
 
 #if 0 /// UNNEEDED by elogind
-static void test_proc_cmdline_override(void) {
-        log_info("/* %s */", __func__);
-
+TEST(proc_cmdline_override) {
         assert_se(putenv((char*) "SYSTEMD_PROC_CMDLINE=foo_bar=quux wuff-piep=tuet zumm some_arg_with_space='foo bar' and_one_more=\"zzz aaa\"") == 0);
         assert_se(putenv((char*) "SYSTEMD_EFI_OPTIONS=different") == 0);
 
@@ -83,12 +79,12 @@ static int parse_item_given(const char *key, const char *value, void *data) {
         else if (in_initrd() && !*strip && proc_cmdline_key_streq(key, "rd.zumm"))
                 assert_se(!value);
         else
-                assert_not_reached("Bad key!");
+                assert_not_reached();
 
         return 0;
 }
 
-static void test_proc_cmdline_given(bool flip_initrd) {
+static void test_proc_cmdline_given_one(bool flip_initrd) {
         log_info("/* %s (flip: %s) */", __func__, yes_no(flip_initrd));
 
         if (flip_initrd)
@@ -105,10 +101,15 @@ static void test_proc_cmdline_given(bool flip_initrd) {
                 in_initrd_force(!in_initrd());
 }
 
-static void test_proc_cmdline_get_key(void) {
+TEST(test_proc_cmdline_given) {
+        test_proc_cmdline_given_one(false);
+        /* Repeat the same thing, but now flip our ininitrdness */
+        test_proc_cmdline_given_one(true);
+}
+
+TEST(proc_cmdline_get_key) {
         _cleanup_free_ char *value = NULL;
 
-        log_info("/* %s */", __func__);
         assert_se(putenv((char*) "SYSTEMD_PROC_CMDLINE=foo_bar=quux wuff-piep=tuet zumm spaaace='ö ü ß' ticks=\"''\"\n\nkkk=uuu\n\n\n") == 0);
 
         assert_se(proc_cmdline_get_key("", 0, &value) == -EINVAL);
@@ -151,10 +152,9 @@ static void test_proc_cmdline_get_key(void) {
         assert_se(proc_cmdline_get_key("kkk", 0, &value) > 0 && streq_ptr(value, "uuu"));
 }
 
-static void test_proc_cmdline_get_bool(void) {
+TEST(proc_cmdline_get_bool) {
         bool value = false;
 
-        log_info("/* %s */", __func__);
         assert_se(putenv((char*) "SYSTEMD_PROC_CMDLINE=foo_bar bar-waldo=1 x_y-z=0 quux=miep\nda=yes\nthe=1") == 0);
         assert_se(putenv((char*) "SYSTEMD_EFI_OPTIONS=") == 0);
 
@@ -171,11 +171,15 @@ static void test_proc_cmdline_get_bool(void) {
         assert_se(proc_cmdline_get_bool("quux", &value) == -EINVAL && value == false);
         assert_se(proc_cmdline_get_bool("da", &value) > 0 && value == true);
         assert_se(proc_cmdline_get_bool("the", &value) > 0 && value == true);
+}
+
+#if ENABLE_EFI
+TEST(proc_cmdline_get_bool_efi) {
+        bool value = false;
 
         assert_se(putenv((char*) "SYSTEMD_PROC_CMDLINE=") == 0);
         assert_se(putenv((char*) "SYSTEMD_EFI_OPTIONS=foo_bar bar-waldo=1 x_y-z=0 quux=miep\nda=yes\nthe=1") == 0);
 
-#if ENABLE_EFI
         assert_se(proc_cmdline_get_bool("", &value) == -EINVAL);
         assert_se(proc_cmdline_get_bool("abc", &value) == 0 && value == false);
         assert_se(proc_cmdline_get_bool("foo_bar", &value) > 0 && value == true);
@@ -189,13 +193,12 @@ static void test_proc_cmdline_get_bool(void) {
         assert_se(proc_cmdline_get_bool("quux", &value) == -EINVAL && value == false);
         assert_se(proc_cmdline_get_bool("da", &value) > 0 && value == true);
         assert_se(proc_cmdline_get_bool("the", &value) > 0 && value == true);
-#endif
 }
+#endif
 
-static void test_proc_cmdline_get_key_many(void) {
+TEST(proc_cmdline_get_key_many) {
         _cleanup_free_ char *value1 = NULL, *value2 = NULL, *value3 = NULL, *value4 = NULL, *value5 = NULL, *value6 = NULL, *value7 = NULL;
 
-        log_info("/* %s */", __func__);
         assert_se(putenv((char*) "SYSTEMD_PROC_CMDLINE=foo_bar=quux wuff-piep=tuet zumm SPACE='one two' doubleticks=\" aaa aaa \"\n\nzummm='\n'\n") == 0);
 
         assert_se(proc_cmdline_get_key_many(0,
@@ -217,9 +220,7 @@ static void test_proc_cmdline_get_key_many(void) {
 }
 #endif // 0
 
-static void test_proc_cmdline_key_streq(void) {
-        log_info("/* %s */", __func__);
-
+TEST(proc_cmdline_key_streq) {
         assert_se(proc_cmdline_key_streq("", ""));
         assert_se(proc_cmdline_key_streq("a", "a"));
         assert_se(!proc_cmdline_key_streq("", "a"));
@@ -235,9 +236,7 @@ static void test_proc_cmdline_key_streq(void) {
 }
 
 #if 0 /// UNNEEDED by elogind
-static void test_proc_cmdline_key_startswith(void) {
-        log_info("/* %s */", __func__);
-
+TEST(proc_cmdline_key_startswith) {
         assert_se(proc_cmdline_key_startswith("", ""));
         assert_se(proc_cmdline_key_startswith("x", ""));
         assert_se(!proc_cmdline_key_startswith("", "x"));
@@ -252,27 +251,11 @@ static void test_proc_cmdline_key_startswith(void) {
 }
 
 #endif // 0
-
-int main(void) {
-        test_setup_logging(LOG_INFO);
-
+static int intro(void) {
         if (access("/proc/cmdline", R_OK) < 0 && ERRNO_IS_PRIVILEGE(errno))
                 return log_tests_skipped("can't read /proc/cmdline");
 
-        test_proc_cmdline_parse();
-#if 0 /// UNNEEDED by elogind
-        test_proc_cmdline_override();
-        test_proc_cmdline_given(false);
-        /* Repeat the same thing, but now flip our ininitrdness */
-        test_proc_cmdline_given(true);
-#endif // 0
-        test_proc_cmdline_key_streq();
-#if 0 /// UNNEEDED by elogind
-        test_proc_cmdline_key_startswith();
-        test_proc_cmdline_get_key();
-        test_proc_cmdline_get_bool();
-        test_proc_cmdline_get_key_many();
-#endif // 0
-
-        return 0;
+        return EXIT_SUCCESS;
 }
+
+DEFINE_TEST_MAIN_WITH_INTRO(LOG_INFO, intro);
