@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #if 0 /// UNNEEDED by elogind (only needed for testing __cpu_mask)
 #include <sched.h>
@@ -18,6 +18,11 @@
 
 DISABLE_WARNING_TYPE_LIMITS;
 
+#define info_no_sign(t)                                                 \
+        printf("%s → %zu bits, %zu byte alignment\n", STRINGIFY(t),     \
+               sizeof(t)*CHAR_BIT,                                      \
+               __alignof__(t))
+
 #define info(t)                                                         \
         printf("%s → %zu bits%s, %zu byte alignment\n", STRINGIFY(t),   \
                sizeof(t)*CHAR_BIT,                                      \
@@ -39,17 +44,25 @@ enum BigEnum2 {
 };
 
 int main(void) {
+        int (*function_pointer)(void);
+
+        info_no_sign(function_pointer);
+        info_no_sign(void*);
+        info(char*);
+
         info(char);
         info(signed char);
         info(unsigned char);
         info(short unsigned);
         info(unsigned);
-        info(long unsigned);
-        info(long long unsigned);
 #ifdef __GLIBC__
+        info(unsigned long);
+        info(unsigned long long);
         info(__syscall_ulong_t);
         info(__syscall_slong_t);
 #endif // ifdef __GLIBC__
+        info(intmax_t);
+        info(uintmax_t);
 
         info(float);
         info(double);
@@ -86,5 +99,15 @@ int main(void) {
         printf("big_enum2_pos → %zu\n", sizeof(big_enum2_pos));
         printf("big_enum2_neg → %zu\n", sizeof(big_enum2_neg));
 
+        printf("timeval: %zu\n", sizeof(struct timeval));
+        printf("timespec: %zu\n", sizeof(struct timespec));
+
+        void *x = malloc(100);
+
+        printf("local variable: %p\n", &function_pointer);
+        printf("glibc function: %p\n", memcpy);
+        printf("heap allocation: %p\n", x);
+        free(x);
+
         return 0;
 }
