@@ -1,4 +1,4 @@
-/* SPDX-License-Identifier: LGPL-2.1+ */
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 
 #include <errno.h>
 //#include <sys/stat.h>
@@ -11,19 +11,19 @@
 
 #include "sd-bus.h"
 
-#include "alloc-util.h"
-#include "bus-error.h"
-#include "bus-util.h"
-#include "format-util.h"
+//#include "alloc-util.h"
+//#include "bus-error.h"
+//#include "bus-util.h"
+//#include "format-util.h"
 #include "log.h"
 #include "macro.h"
-#include "main-func.h"
-#include "process-util.h"
-#include "special.h"
+//#include "main-func.h"
+//#include "process-util.h"
+//#include "special.h"
 #include "stdio-util.h"
-#include "strv.h"
-#include "unit-name.h"
-#include "util.h"
+//#include "strv.h"
+//#include "unit-name.h"
+//#include "util.h"
 #include "utmp-wtmp.h"
 
 /// Additional includes needed by elogind
@@ -49,7 +49,7 @@ static void context_clear(Context *c) {
 }
 
 #if 0 /// UNNEEDED by elogind
-static usec_t get_startup_time(Context *c) {
+static usec_t get_startup_monotonic_time(Context *c) {
         _cleanup_(sd_bus_error_free) sd_bus_error error = SD_BUS_ERROR_NULL;
         usec_t t = 0;
         int r;
@@ -61,7 +61,7 @@ static usec_t get_startup_time(Context *c) {
                         "org.freedesktop.systemd1",
                         "/org/freedesktop/systemd1",
                         "org.freedesktop.systemd1.Manager",
-                        "UserspaceTimestamp",
+                        "UserspaceTimestampMonotonic",
                         &error,
                         't', &t);
         if (r < 0) {
@@ -121,6 +121,7 @@ static int get_current_runlevel(Context *c) {
 static int on_reboot(Context *c) {
         int r = 0, q;
         usec_t t;
+        usec_t boottime;
 
         assert(c);
 
@@ -137,16 +138,21 @@ static int on_reboot(Context *c) {
 #if 0 /// systemd hasn't started the system, so elogind always uses NOW()
         /* If this call fails it will return 0, which
          * utmp_put_reboot() will then fix to the current time */
-        t = get_startup_time(c);
+        t = get_startup_monotonic_time(c);
 #else // 0
         t = now(CLOCK_REALTIME);
 #endif // 0
 
-        q = utmp_put_reboot(t);
-        if (q < 0) {
-                log_error_errno(q, "Failed to write utmp record: %m");
-                r = q;
-        }
+        boottime = map_clock_usec(t, CLOCK_MONOTONIC, CLOCK_REALTIME);
+        /* We query the recorded monotonic time here (instead of the system clock CLOCK_REALTIME),
+         * even though we actually want the system clock time. That's because there's a likely
+         * chance that the system clock wasn't set right during early boot. By manually converting
+         * the monotonic clock to the system clock here we can compensate
+         * for incorrectly set clocks during early boot. */
+
+        q = utmp_put_reboot(boottime);
+        if (q < 0)
+                r = log_error_errno(q, "Failed to write utmp record: %m");
 
         return r;
 }
@@ -167,10 +173,8 @@ static int on_shutdown(Context *c) {
 #endif
 
         q = utmp_put_shutdown();
-        if (q < 0) {
-                log_error_errno(q, "Failed to write utmp record: %m");
-                r = q;
-        }
+        if (q < 0)
+                r = log_error_errno(q, "Failed to write utmp record: %m");
 
         return r;
 }
@@ -235,7 +239,7 @@ void update_utmp(int argc, char* argv[]) {
 #endif // 0
         _cleanup_(context_clear) Context c = {
 #if HAVE_AUDIT
-                .audit_fd = -1
+                .audit_fd = -1,
 #endif
         };
 #if 0 /// UNNEEDED by elogind
@@ -245,7 +249,7 @@ void update_utmp(int argc, char* argv[]) {
                 return log_error_errno(SYNTHETIC_ERRNO(EINVAL),
                                        "This program requires one argument.");
 
-        log_setup_service();
+        log_setup();
 
         umask(0022);
 #else // 0
